@@ -1,26 +1,29 @@
 /*
  *   moohtml.c
  *
- *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2010 by Yevgen Muntyan <emuntyan@users.sourceforge.net>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 #include "moohtml.h"
 #include "marshals.h"
 #include "mooutils/mooutils-misc.h"
+#include "mooutils/moocompat.h"
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 #include <glib/gregex.h>
 #include <string.h>
 #include <errno.h>
 
-#if defined(MOO_USE_XML)
 #include <libxml/HTMLtree.h>
 
 #define DEFAULT_PAR_SPACING 6
@@ -206,7 +209,7 @@ _moo_html_tag_class_init (MooHtmlTagClass *klass)
                                              "href",
                                              "href",
                                              NULL,
-                                             G_PARAM_READWRITE));
+                                             (GParamFlags) G_PARAM_READWRITE));
 }
 
 
@@ -302,7 +305,7 @@ _moo_html_class_init (MooHtmlClass *klass)
                                              "title",
                                              "title",
                                              NULL,
-                                             G_PARAM_READWRITE));
+                                             (GParamFlags) G_PARAM_READWRITE));
 
     g_object_class_install_property (gobject_class,
                                      HTML_PROP_MARKUP,
@@ -521,7 +524,7 @@ moo_html_load_url (GtkTextView *view,
 
     if (!moo_html_parse_url (url, &scheme, &base, &anchor))
     {
-        g_warning ("%s: invalid url '%s'", G_STRLOC, url);
+        g_warning ("invalid url '%s'", url);
         return FALSE;
     }
 
@@ -656,7 +659,7 @@ _moo_html_load_memory (GtkTextView *view,
 
     if (!doc)
     {
-        g_warning ("%s: htmlReadMemory failed", G_STRLOC);
+        g_warning ("htmlReadMemory failed");
         return FALSE;
     }
 
@@ -985,7 +988,7 @@ moo_html_create_tag (GtkTextView       *view,
         real_attr = *attr;
     }
 
-    tag = g_object_new (MOO_TYPE_HTML_TAG, NULL);
+    tag = g_object_new (MOO_TYPE_HTML_TAG, (const char*) NULL);
     gtk_text_tag_table_add (gtk_text_buffer_get_tag_table (gtk_text_view_get_buffer (view)),
                             GTK_TEXT_TAG (tag));
     g_object_unref (tag);
@@ -1258,8 +1261,8 @@ moo_html_goto_anchor (GtkTextView *view,
 
     if (!mark)
     {
-        g_warning ("%s: could not find anchor '%s'",
-                   G_STRLOC, anchor);
+        g_warning ("could not find anchor '%s'",
+                   anchor);
         return FALSE;
     }
     else
@@ -1408,7 +1411,7 @@ moo_html_load_doc (GtkTextView *view,
         }
         else
         {
-            g_warning ("%s: unknown node '%s'", G_STRLOC, node->name);
+            g_warning ("unknown node '%s'", node->name);
         }
     }
 }
@@ -1436,7 +1439,7 @@ moo_html_load_head (GtkTextView *view,
         }
         else
         {
-            g_message ("%s: unknown node '%s'", G_STRLOC, child->name);
+            g_message ("unknown node '%s'", child->name);
         }
     }
 }
@@ -1842,7 +1845,7 @@ process_elm_body (GtkTextView    *view,
 
         else if (IS_ELEMENT (child))
         {
-            g_message ("%s: unknown node '%s'", G_STRLOC, child->name);
+            g_message ("unknown node '%s'", child->name);
             process_elm_body (view, buffer, child, current, iter);
         }
         else if (IS_COMMENT (child))
@@ -1851,7 +1854,7 @@ process_elm_body (GtkTextView    *view,
         }
         else
         {
-            g_warning ("%s: unknown node", G_STRLOC);
+            g_warning ("unknown node");
         }
     }
 }
@@ -1961,8 +1964,11 @@ parse_int (const char *str,
 
     if (errno)
         return FALSE;
+
+#if G_MAXINT != G_MAXLONG
     if (num < G_MININT || num > G_MAXINT)
         return FALSE;
+#endif
 
     if (dest)
         *dest = num;
@@ -1989,7 +1995,7 @@ make_li_number (int     count,
     {
         case OL_UPPER_ROMAN:
         case OL_LOWER_ROMAN:
-            g_warning ("%s: implement me", G_STRLOC);
+            g_warning ("implement me");
         case OL_NUM:
             return g_strdup_printf (" %d. ", count);
         case OL_LOWER_ALPHA:
@@ -2034,8 +2040,8 @@ process_ol_elm (GtkTextView    *view,
             list_type = OL_UPPER_ROMAN;
         else
         {
-            g_warning ("%s: invalid type attribute '%s'",
-                       G_STRLOC, type);
+            g_warning ("invalid type attribute '%s'",
+                       type);
         }
     }
 
@@ -2066,7 +2072,7 @@ process_ol_elm (GtkTextView    *view,
         }
         else
         {
-            g_message ("%s: unknown node '%s'", G_STRLOC, child->name);
+            g_message ("unknown node '%s'", child->name);
             process_elm_body (view, buffer, child, current, iter);
         }
     }
@@ -2165,7 +2171,7 @@ process_font_elm (GtkTextView   *view,
 
         if (!size[0] || size[0] < '1' || size[0] > '7')
         {
-            g_warning ("%s: invalid size '%s'", G_STRLOC, size);
+            g_warning ("invalid size '%s'", size);
         }
         else
         {
@@ -2361,9 +2367,7 @@ process_img_elm (GtkTextView    *view,
 
     if (!pixbuf)
     {
-        g_message ("%s: could not load image '%s'",
-                   G_STRLOC, path);
-        g_message ("%s: %s", G_STRLOC, error->message);
+        g_message ("could not load image '%s': %s", path, moo_error_message (error));
         g_error_free (error);
         goto try_alt;
     }
@@ -2411,18 +2415,3 @@ process_tr_elm (GtkTextView *view, GtkTextBuffer *buffer, xmlNode *elm,
     process_elm_body (view, buffer, elm, parent, iter);
     moo_html_new_line (view, buffer, iter, parent, FALSE);
 }
-
-#else /* !MOO_USE_XML */
-void
-moo_text_view_set_markup (GtkTextView *view,
-                          const char  *markup)
-{
-    GtkTextBuffer *buffer;
-
-    g_return_if_fail (GTK_IS_TEXT_VIEW (view));
-    g_return_if_fail (markup != NULL);
-
-    buffer = gtk_text_view_get_buffer (view);
-    gtk_text_buffer_set_text (buffer, markup, -1);
-}
-#endif /* MOO_USE_XML */
