@@ -1,63 +1,64 @@
 /*
  *   mooeditdialogs.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2010 by Yevgen Muntyan <emuntyan@users.sourceforge.net>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #define MOOEDIT_COMPILATION
 #include "mooedit/mooeditdialogs.h"
-#include "mooedit/mooedit-private.h"
 #include "mooedit/mooeditprefs.h"
-#include "mooedit/mooeditfileops.h"
+#include "mooedit/mooedit-fileops.h"
+#include "mooedit/mooeditfileinfo.h"
 #include "mooutils/moodialogs.h"
-#include "mooutils/moocompat.h"
 #include "mooutils/moostock.h"
 #include "mooutils/mooi18n.h"
 #include "mooutils/mooencodings.h"
-#include "glade/mootextfind-prompt-gxml.h"
-#include "glade/mooeditsavemult-gxml.h"
+#include "mooutils/mooutils.h"
+#include "mooedit/mootextfind-prompt-gxml.h"
+#include "mooedit/mooeditsavemult-gxml.h"
+#include "mooedit/mootryencoding-gxml.h"
 #include <gtk/gtk.h>
 #include <glib/gregex.h>
 #include <string.h>
 
 
-GSList *
+MooOpenInfoArray *
 _moo_edit_open_dialog (GtkWidget *widget,
                        MooEdit   *current_doc)
 {
     MooFileDialog *dialog;
-    const char *start_dir = NULL;
     const char *encoding;
-    char *new_start;
-    char *freeme = NULL;
-    char **filenames = NULL, **p;
-    GSList *infos = NULL;
+    GFile *start = NULL;
+    MooFileArray *files = NULL;
+    MooOpenInfoArray *info_array = NULL;
+    guint i;
 
     moo_prefs_create_key (moo_edit_setting (MOO_EDIT_PREFS_LAST_DIR), MOO_PREFS_STATE, G_TYPE_STRING, NULL);
 
     if (current_doc && moo_prefs_get_bool (moo_edit_setting (MOO_EDIT_PREFS_DIALOGS_OPEN_FOLLOWS_DOC)))
     {
-        char *filename = moo_edit_get_filename (current_doc);
+        GFile *file = moo_edit_get_file (current_doc);
 
-        if (filename)
-        {
-            freeme = g_path_get_dirname (filename);
-            start_dir = freeme;
-            g_free (filename);
-        }
+        if (file)
+            start = g_file_get_parent (file);
+
+        g_object_unref (file);
     }
 
-    if (!start_dir)
-        start_dir = moo_prefs_get_filename (moo_edit_setting (MOO_EDIT_PREFS_LAST_DIR));
+    if (!start)
+        start = moo_prefs_get_file (moo_edit_setting (MOO_EDIT_PREFS_LAST_DIR));
 
     dialog = moo_file_dialog_new (MOO_FILE_DIALOG_OPEN, widget,
-                                  TRUE, GTK_STOCK_OPEN, start_dir,
+                                  TRUE, GTK_STOCK_OPEN, start,
                                   NULL);
     g_object_set (dialog, "enable-encodings", TRUE, NULL);
     moo_file_dialog_set_help_id (dialog, "dialog-open");
@@ -65,67 +66,65 @@ _moo_edit_open_dialog (GtkWidget *widget,
 
     moo_file_dialog_set_filter_mgr_id (dialog, "MooEdit");
 
-    if (!moo_file_dialog_run (dialog))
-        goto out;
+    if (moo_file_dialog_run (dialog))
+    {
+        encoding = moo_file_dialog_get_encoding (dialog);
 
-    encoding = moo_file_dialog_get_encoding (dialog);
+        if (encoding && !strcmp (encoding, MOO_ENCODING_AUTO))
+            encoding = NULL;
 
-    if (encoding && !strcmp (encoding, MOO_ENCODING_AUTO))
-        encoding = NULL;
+        files = moo_file_dialog_get_files (dialog);
+        g_return_val_if_fail (files != NULL && files->n_elms != 0, NULL);
 
-    filenames = moo_file_dialog_get_filenames (dialog);
-    g_return_val_if_fail (filenames != NULL, NULL);
+        info_array = moo_open_info_array_new ();
+        for (i = 0; i < files->n_elms; ++i)
+            moo_open_info_array_take (info_array, moo_open_info_new_file (files->elms[i], encoding, -1, 0));
 
-    for (p = filenames; *p != NULL; ++p)
-        infos = g_slist_prepend (infos, moo_edit_file_info_new (*p, encoding));
-    infos = g_slist_reverse (infos);
+        g_object_unref (start);
+        start = g_file_get_parent (files->elms[0]);
+        moo_prefs_set_file (moo_edit_setting (MOO_EDIT_PREFS_LAST_DIR), start);
+    }
 
-    new_start = g_path_get_dirname (filenames[0]);
-    moo_prefs_set_filename (moo_edit_setting (MOO_EDIT_PREFS_LAST_DIR), new_start);
-    g_free (new_start);
-
-out:
-    g_free (freeme);
+    g_object_unref (start);
     g_object_unref (dialog);
-    g_strfreev (filenames);
-    return infos;
+    moo_file_array_free (files);
+    return info_array;
 }
 
 
-MooEditFileInfo *
-_moo_edit_save_as_dialog (MooEdit    *edit,
+MooSaveInfo *
+_moo_edit_save_as_dialog (MooEdit    *doc,
                           const char *display_basename)
 {
-    const char *start = NULL;
-    const char *filename = NULL;
-    char *freeme = NULL;
     const char *encoding;
-    char *new_start;
     MooFileDialog *dialog;
-    MooEditFileInfo *file_info;
+    MooSaveInfo *info;
+    GFile *start = NULL;
+    GFile *file = NULL;
+
+    g_return_val_if_fail (MOO_IS_EDIT (doc), NULL);
 
     moo_prefs_create_key (moo_edit_setting (MOO_EDIT_PREFS_LAST_DIR),
                           MOO_PREFS_STATE, G_TYPE_STRING, NULL);
 
     if (moo_prefs_get_bool (moo_edit_setting (MOO_EDIT_PREFS_DIALOGS_OPEN_FOLLOWS_DOC)))
     {
-        char *this_filename = moo_edit_get_filename (edit);
-
-        if (this_filename)
-        {
-            freeme = g_path_get_dirname (this_filename);
-            start = freeme;
-            g_free (this_filename);
-        }
+        file = moo_edit_get_file (doc);
+        if (file)
+            start = g_file_get_parent (file);
+        g_object_unref (file);
+        file = NULL;
     }
 
     if (!start)
-        start = moo_prefs_get_filename (moo_edit_setting (MOO_EDIT_PREFS_LAST_DIR));
+        start = moo_prefs_get_file (moo_edit_setting (MOO_EDIT_PREFS_LAST_DIR));
 
-    dialog = moo_file_dialog_new (MOO_FILE_DIALOG_SAVE, GTK_WIDGET (edit),
-                                  FALSE, GTK_STOCK_SAVE_AS, start, display_basename);
+    dialog = moo_file_dialog_new (MOO_FILE_DIALOG_SAVE,
+                                  GTK_WIDGET (moo_edit_get_view (doc)),
+                                  FALSE, GTK_STOCK_SAVE_AS,
+                                  start, display_basename);
     g_object_set (dialog, "enable-encodings", TRUE, NULL);
-    moo_file_dialog_set_encoding (dialog, moo_edit_get_encoding (edit));
+    moo_file_dialog_set_encoding (dialog, moo_edit_get_encoding (doc));
     moo_file_dialog_set_help_id (dialog, "dialog-save");
 
     moo_file_dialog_set_filter_mgr_id (dialog, "MooEdit");
@@ -133,31 +132,32 @@ _moo_edit_save_as_dialog (MooEdit    *edit,
     if (!moo_file_dialog_run (dialog))
     {
         g_object_unref (dialog);
-        g_free (freeme);
+        g_object_unref (start);
         return NULL;
     }
 
     encoding = moo_file_dialog_get_encoding (dialog);
-    filename = moo_file_dialog_get_filename (dialog);
-    g_return_val_if_fail (filename != NULL, NULL);
-    file_info = moo_edit_file_info_new (filename, encoding);
+    file = moo_file_dialog_get_file (dialog);
+    g_return_val_if_fail (file != NULL, NULL);
+    info = moo_save_info_new_file (file, encoding);
 
-    new_start = g_path_get_dirname (filename);
-    moo_prefs_set_filename (moo_edit_setting (MOO_EDIT_PREFS_LAST_DIR), new_start);
-    g_free (new_start);
+    g_object_unref (start);
+    start = g_file_get_parent (file);
+    moo_prefs_set_file (moo_edit_setting (MOO_EDIT_PREFS_LAST_DIR), start);
 
+    g_object_unref (start);
+    g_object_unref (file);
     g_object_unref (dialog);
-    g_free (freeme);
-    return file_info;
+    return info;
 }
 
 
-MooSaveChangesDialogResponse
-_moo_edit_save_changes_dialog (MooEdit *edit)
+MooSaveChangesResponse
+_moo_edit_save_changes_dialog (MooEdit *doc)
 {
-    g_return_val_if_fail (MOO_IS_EDIT (edit), MOO_SAVE_CHANGES_RESPONSE_CANCEL);
-    return moo_save_changes_dialog (moo_edit_get_display_basename (edit),
-                                    GTK_WIDGET (edit));
+    g_return_val_if_fail (MOO_IS_EDIT (doc), MOO_SAVE_CHANGES_RESPONSE_CANCEL);
+    return moo_save_changes_dialog (moo_edit_get_display_basename (doc),
+                                    GTK_WIDGET (moo_edit_get_view (doc)));
 }
 
 
@@ -177,13 +177,13 @@ name_data_func (G_GNUC_UNUSED GtkTreeViewColumn *column,
                 GtkTreeModel      *model,
                 GtkTreeIter       *iter)
 {
-    MooEdit *edit = NULL;
+    MooEdit *doc = NULL;
 
-    gtk_tree_model_get (model, iter, COLUMN_EDIT, &edit, -1);
-    g_return_if_fail (MOO_IS_EDIT (edit));
+    gtk_tree_model_get (model, iter, COLUMN_EDIT, &doc, -1);
+    g_return_if_fail (MOO_IS_EDIT (doc));
 
-    g_object_set (cell, "text", moo_edit_get_display_basename (edit), NULL);
-    g_object_unref (edit);
+    g_object_set (cell, "text", moo_edit_get_display_basename (doc), NULL);
+    g_object_unref (doc);
 }
 
 
@@ -242,22 +242,23 @@ save_toggled (GtkCellRendererToggle *cell,
 }
 
 static void
-files_treeview_init (GtkTreeView *treeview, GtkWidget *dialog, GSList  *docs)
+files_treeview_init (GtkTreeView *treeview, GtkWidget *dialog, MooEditArray *docs)
 {
     GtkListStore *store;
     GtkTreeViewColumn *column;
     GtkCellRenderer *cell;
-    GSList *l;
+    guint i;
 
     store = gtk_list_store_new (NUM_COLUMNS, G_TYPE_BOOLEAN, MOO_TYPE_EDIT);
 
-    for (l = docs; l != NULL; l = l->next)
+    for (i = 0; i < docs->n_elms; ++i)
     {
         GtkTreeIter iter;
         gtk_list_store_append (store, &iter);
         gtk_list_store_set (store, &iter,
                             COLUMN_SAVE, TRUE,
-                            COLUMN_EDIT, l->data, -1);
+                            COLUMN_EDIT, docs->elms[i],
+                            -1);
     }
 
     gtk_tree_view_set_model (treeview, GTK_TREE_MODEL (store));
@@ -284,61 +285,75 @@ files_treeview_init (GtkTreeView *treeview, GtkWidget *dialog, GSList  *docs)
 }
 
 
-static GSList *
-files_treeview_get_to_save (GtkTreeView *treeview)
+static void
+files_treeview_get_to_save (GtkTreeView  *treeview,
+                            MooEditArray *to_save)
 {
     GtkTreeIter iter;
     GtkTreeModel *model;
-    GSList *list = NULL;
 
     model = gtk_tree_view_get_model (treeview);
-    g_return_val_if_fail (model != NULL, NULL);
+    g_return_if_fail (model != NULL);
 
     gtk_tree_model_get_iter_first (model, &iter);
 
     do
     {
-        MooEdit *edit = NULL;
+        MooEdit *doc = NULL;
         gboolean save = TRUE;
 
         gtk_tree_model_get (model, &iter,
                             COLUMN_SAVE, &save,
-                            COLUMN_EDIT, &edit, -1);
-        g_return_val_if_fail (MOO_IS_EDIT (edit), list);
+                            COLUMN_EDIT, &doc, -1);
+        g_return_if_fail (MOO_IS_EDIT (doc));
 
         if (save)
-            list = g_slist_prepend (list, edit);
+            moo_edit_array_append (to_save, doc);
 
-        g_object_unref (edit);
+        g_object_unref (doc);
     }
     while (gtk_tree_model_iter_next (model, &iter));
-
-    return g_slist_reverse (list);
 }
 
 
-MooSaveChangesDialogResponse
-_moo_edit_save_multiple_changes_dialog (GSList  *docs,
-                                        GSList **to_save)
+static GtkWidget *
+find_widget_for_response (GtkDialog *dialog,
+                          int        response)
+{
+    GList *l, *children;
+    GtkWidget *ret = NULL;
+
+    children = gtk_container_get_children (GTK_CONTAINER (dialog->action_area));
+
+    for (l = children; ret == NULL && l != NULL; l = l->next)
+    {
+        GtkWidget *widget = l->data;
+        int response_here = gtk_dialog_get_response_for_widget (dialog, widget);
+        if (response_here == response)
+            ret = widget;
+    }
+
+    g_list_free (children);
+    return ret;
+}
+
+MooSaveChangesResponse
+_moo_edit_save_multiple_changes_dialog (MooEditArray *docs,
+                                        MooEditArray *to_save)
 {
-    GSList *l;
     GtkWidget *dialog;
     char *msg, *question;
     int response;
-    MooSaveChangesDialogResponse retval;
+    MooSaveChangesResponse retval;
     SaveMultDialogXml *xml;
 
-    g_return_val_if_fail (docs != NULL, MOO_SAVE_CHANGES_RESPONSE_CANCEL);
-    g_return_val_if_fail (docs->next != NULL, MOO_SAVE_CHANGES_RESPONSE_CANCEL);
+    g_return_val_if_fail (docs != NULL && docs->n_elms > 1, MOO_SAVE_CHANGES_RESPONSE_CANCEL);
     g_return_val_if_fail (to_save != NULL, MOO_SAVE_CHANGES_RESPONSE_CANCEL);
 
-    for (l = docs; l != NULL; l = l->next)
-        g_return_val_if_fail (MOO_IS_EDIT (l->data), MOO_SAVE_CHANGES_RESPONSE_CANCEL);
-
     xml = save_mult_dialog_xml_new ();
     dialog = GTK_WIDGET (xml->SaveMultDialog);
 
-    moo_window_set_parent (dialog, docs->data);
+    moo_window_set_parent (dialog, GTK_WIDGET (moo_edit_get_view (docs->elms[0])));
 
     gtk_dialog_add_buttons (GTK_DIALOG (dialog),
                             MOO_STOCK_SAVE_NONE, GTK_RESPONSE_NO,
@@ -360,14 +375,20 @@ _moo_edit_save_multiple_changes_dialog (GSList  *docs,
                                             "Save changes before closing?",
                                            "There are %u documents with unsaved changes. "
                                             "Save changes before closing?",
-                                           g_slist_length (docs)),
-                                g_slist_length (docs));
+                                           docs->n_elms),
+                                (guint) docs->n_elms);
     msg = g_markup_printf_escaped ("<span weight=\"bold\" size=\"larger\">%s</span>",
                                    question);
     gtk_label_set_markup (xml->label, msg);
 
     files_treeview_init (xml->treeview, dialog, docs);
 
+    {
+        GtkWidget *button;
+        button = find_widget_for_response (GTK_DIALOG (dialog), GTK_RESPONSE_YES);
+        gtk_widget_grab_focus (button);
+    }
+
     response = gtk_dialog_run (GTK_DIALOG (dialog));
 
     switch (response)
@@ -376,7 +397,7 @@ _moo_edit_save_multiple_changes_dialog (GSList  *docs,
             retval = MOO_SAVE_CHANGES_RESPONSE_DONT_SAVE;
             break;
         case GTK_RESPONSE_YES:
-            *to_save = files_treeview_get_to_save (xml->treeview);
+            files_treeview_get_to_save (xml->treeview, to_save);
             retval = MOO_SAVE_CHANGES_RESPONSE_SAVE;
             break;
         default:
@@ -395,152 +416,40 @@ _moo_edit_save_multiple_changes_dialog (GSList  *docs,
  */
 
 void
-_moo_edit_save_error_dialog (GtkWidget      *widget,
-                             const char     *filename,
-                             GError         *error)
+_moo_edit_save_error_dialog (MooEdit *doc,
+                             GFile   *file,
+                             GError  *error)
 {
-    char *filename_utf8, *msg = NULL;
+    char *filename, *msg = NULL;
 
-    g_return_if_fail (filename != NULL);
+    g_return_if_fail (G_IS_FILE (file));
 
-    filename_utf8 = g_filename_to_utf8 (filename, -1, NULL, NULL, NULL);
+    filename = moo_file_get_display_name (file);
 
-    if (!filename_utf8)
-        g_critical ("%s: could not convert filename '%s' to utf8", G_STRLOC, filename);
+    msg = g_strdup_printf (_("Could not save file\n%s"), filename);
 
-    if (filename_utf8)
-        /* Could not save file foo.txt */
-        msg = g_strdup_printf (_("Could not save file\n%s"), filename_utf8);
-    else
-        msg = g_strdup (_("Could not save file"));
-
-    moo_error_dialog (widget, msg, error ? error->message : NULL);
+    moo_error_dialog (msg, moo_error_message (error),
+                      GTK_WIDGET (moo_edit_get_view (doc)));
 
     g_free (msg);
-    g_free (filename_utf8);
+    g_free (filename);
 }
 
-void
-_moo_edit_save_error_enc_dialog (GtkWidget  *widget,
-                                 const char *filename,
-                                 const char *encoding)
-{
-    char *filename_utf8, *msg = NULL;
-    char *secondary;
-
-    g_return_if_fail (filename != NULL);
-    g_return_if_fail (encoding != NULL);
-
-    filename_utf8 = g_filename_to_utf8 (filename, -1, NULL, NULL, NULL);
-
-    if (!filename_utf8)
-        g_critical ("%s: could not convert filename '%s' to utf8", G_STRLOC, filename);
-
-    if (filename_utf8)
-        /* Error saving file foo.txt */
-        msg = g_strdup_printf (_("Error saving file\n%s"), filename_utf8);
-    else
-        msg = g_strdup (_("Error saving file"));
-
-    secondary = g_strdup_printf (_("Could not convert file to requested character "
-                                   "encoding %s. File was saved in UTF-8 encoding."),
-                                 encoding);
-
-    moo_error_dialog (widget, msg, secondary);
-
-    g_free (msg);
-    g_free (secondary);
-    g_free (filename_utf8);
-}
-
-
-void
-_moo_edit_open_error_dialog (GtkWidget      *widget,
-                             const char     *filename,
-                             const char     *encoding,
-                             GError         *error)
-{
-    char *filename_utf8, *msg = NULL;
-    char *secondary;
-
-    g_return_if_fail (filename != NULL);
-
-    filename_utf8 = g_filename_to_utf8 (filename, -1, NULL, NULL, NULL);
-
-    if (!filename_utf8)
-        g_critical ("%s: could not convert filename '%s' to utf8", G_STRLOC, filename);
-
-    if (filename_utf8)
-        /* Could not open file foo.txt */
-        msg = g_strdup_printf (_("Could not open file\n%s"), filename_utf8);
-    else
-        msg = g_strdup (_("Could not open file"));
-
-    if (error && error->domain == MOO_EDIT_FILE_ERROR &&
-        error->code == MOO_EDIT_FILE_ERROR_ENCODING)
-    {
-        if (encoding)
-            secondary = g_strdup_printf (_("Could not open file using character encoding %s. "
-                                           "The file may be binary or encoding may be specified "
-                                           "incorrectly."), encoding);
-        else
-            secondary = g_strdup_printf (_("Could not detect file character encoding. "
-                                           "Please make sure the file is not binary and try to select "
-                                           "encoding in the Open dialog."));
-    }
-    else
-    {
-        secondary = error ? g_strdup (error->message) : NULL;
-    }
-
-    moo_error_dialog (widget, msg, secondary);
-
-    g_free (msg);
-    g_free (secondary);
-    g_free (filename_utf8);
-}
-
-
-void
-_moo_edit_reload_error_dialog (MooEdit *doc,
-                               GError  *error)
-{
-    const char *filename;
-    char *msg = NULL;
-
-    g_return_if_fail (MOO_IS_EDIT (doc));
-
-    filename = moo_edit_get_display_basename (doc);
-
-    if (!filename)
-    {
-        g_critical ("%s: oops", G_STRLOC);
-        filename = "";
-    }
-
-    /* Could not reload file foo.txt */
-    msg = g_strdup_printf (_("Could not reload file\n%s"), filename);
-    /* XXX */
-    moo_error_dialog (GTK_WIDGET (doc), msg, error ? error->message : NULL);
-
-    g_free (msg);
-}
-
-
-/*****************************************************************************/
-/* Confirmation and alerts
- */
-
 static gboolean
-moo_edit_question_dialog (MooEdit    *edit,
+moo_edit_question_dialog (MooEdit    *doc,
                           const char *text,
                           const char *secondary,
-                          const char *button)
+                          const char *button,
+                          int         default_response)
 {
     int res;
-    GtkWindow *parent = GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (edit)));
+    MooEditView *view;
+    GtkWindow *parent;
     GtkWidget *dialog;
 
+    view = doc ? moo_edit_get_view (doc) : NULL;
+    parent = view ? GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (view))) : NULL;
+
     dialog = gtk_message_dialog_new (parent, GTK_DIALOG_MODAL,
                                      GTK_MESSAGE_WARNING,
                                      GTK_BUTTONS_NONE,
@@ -557,6 +466,7 @@ moo_edit_question_dialog (MooEdit    *edit,
                                              GTK_RESPONSE_YES,
                                              GTK_RESPONSE_CANCEL,
                                              -1);
+    gtk_dialog_set_default_response (GTK_DIALOG (dialog), default_response);
 
     res = gtk_dialog_run (GTK_DIALOG (dialog));
     gtk_widget_destroy (dialog);
@@ -565,48 +475,206 @@ moo_edit_question_dialog (MooEdit    *edit,
 }
 
 gboolean
-_moo_edit_reload_modified_dialog (MooEdit    *edit)
+_moo_edit_save_error_enc_dialog (MooEdit    *doc,
+                                 GFile      *file,
+                                 const char *encoding)
+{
+    char *filename;
+    char *secondary;
+    gboolean result;
+
+    g_return_val_if_fail (G_IS_FILE (file), FALSE);
+    g_return_val_if_fail (encoding != NULL, FALSE);
+
+    filename = moo_file_get_display_name (file);
+
+    secondary = g_strdup_printf (_("Could not save file %s in encoding %s. "
+                                   "Do you want to save it in UTF-8 encoding instead?"),
+                                 filename, encoding);
+
+    result = moo_edit_question_dialog (doc, _("Save file in UTF-8 encoding?"),
+                                       secondary, GTK_STOCK_OK, GTK_RESPONSE_YES);
+
+    g_free (secondary);
+    g_free (filename);
+    return result;
+}
+
+
+MooEditTryEncodingResponse
+_moo_edit_try_encoding_dialog (G_GNUC_UNUSED GFile *file,
+                               const char  *encoding,
+                               char       **new_encoding)
+{
+    MooEditWindow *window;
+    GtkWidget *dialog;
+    TryEncodingDialogXml *xml;
+    int dialog_response;
+    char *filename = NULL;
+    char *msg = NULL;
+    char *secondary = NULL;
+
+    filename = moo_file_get_display_name (file);
+
+    if (filename)
+    {
+        /* Could not open file foo.txt */
+        char *tmp = g_strdup_printf (_("Could not open file\n%s"), filename);
+        msg = g_markup_printf_escaped ("<b><big>%s</big></b>", tmp);
+        g_free (tmp);
+    }
+    else
+    {
+        const char *tmp = _("Could not open file");
+        msg = g_markup_printf_escaped ("<b><big>%s</big></b>", tmp);
+    }
+
+    if (encoding)
+        secondary = g_strdup_printf (_("Could not open file using character encoding %s. "
+                                       "Try to select another encoding below."), encoding);
+    else
+        secondary = g_strdup_printf (_("Could not detect file character encoding. "
+                                       "Try to select an encoding below."));
+
+    xml = try_encoding_dialog_xml_new ();
+    g_return_val_if_fail (xml && xml->TryEncodingDialog, MOO_EDIT_TRY_ENCODING_RESPONSE_CANCEL);
+
+    gtk_label_set_markup (xml->label_primary, msg);
+    gtk_label_set_text (xml->label_secondary, secondary);
+
+    dialog = GTK_WIDGET (xml->TryEncodingDialog);
+
+    _moo_encodings_combo_init (GTK_COMBO_BOX (xml->encoding_combo), MOO_ENCODING_COMBO_OPEN, FALSE);
+    _moo_encodings_combo_set_enc (GTK_COMBO_BOX (xml->encoding_combo), encoding, MOO_ENCODING_COMBO_OPEN);
+
+    if ((window = moo_editor_get_active_window (moo_editor_instance ())))
+        moo_window_set_parent (dialog, GTK_WIDGET (window));
+
+    gtk_dialog_add_buttons (GTK_DIALOG (dialog),
+                            GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                            GTK_STOCK_OK, GTK_RESPONSE_OK,
+                            NULL);
+    gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
+    gtk_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
+                                             GTK_RESPONSE_OK,
+                                             GTK_RESPONSE_CANCEL,
+                                             -1);
+
+    dialog_response = gtk_dialog_run (GTK_DIALOG (dialog));
+
+    *new_encoding = g_strdup (_moo_encodings_combo_get_enc (GTK_COMBO_BOX (xml->encoding_combo), MOO_ENCODING_COMBO_OPEN));
+
+    gtk_widget_destroy (dialog);
+    g_free (secondary);
+    g_free (msg);
+
+    return dialog_response == GTK_RESPONSE_OK ?
+        MOO_EDIT_TRY_ENCODING_RESPONSE_TRY_ANOTHER :
+        MOO_EDIT_TRY_ENCODING_RESPONSE_CANCEL;
+}
+
+
+void
+_moo_edit_open_error_dialog (GtkWidget  *widget,
+                             GFile      *file,
+                             GError     *error)
+{
+    char *filename, *msg = NULL;
+    char *secondary;
+
+    filename = moo_file_get_display_name (file);
+
+    if (filename)
+        /* Could not open file foo.txt */
+        msg = g_strdup_printf (_("Could not open file\n%s"), filename);
+    else
+        msg = g_strdup (_("Could not open file"));
+
+    secondary = error ? g_strdup (error->message) : NULL;
+
+    moo_error_dialog (msg, secondary, widget);
+
+    g_free (msg);
+    g_free (secondary);
+    g_free (filename);
+}
+
+
+void
+_moo_edit_reload_error_dialog (MooEdit *doc,
+                               GError  *error)
+{
+    const char *filename;
+    char *msg = NULL;
+
+    g_return_if_fail (MOO_IS_EDIT (doc));
+
+    filename = moo_edit_get_display_basename (doc);
+
+    if (!filename)
+    {
+        g_critical ("oops");
+        filename = "";
+    }
+
+    /* Could not reload file foo.txt */
+    msg = g_strdup_printf (_("Could not reload file\n%s"), filename);
+    /* XXX */
+    moo_error_dialog (msg, moo_error_message (error),
+                      GTK_WIDGET (moo_edit_get_view (doc)));
+
+    g_free (msg);
+}
+
+
+/*****************************************************************************/
+/* Confirmation and alerts
+ */
+
+gboolean
+_moo_edit_reload_modified_dialog (MooEdit *doc)
 {
     const char *name;
     char *question;
     gboolean result;
 
-    name = moo_edit_get_display_basename (edit);
+    name = moo_edit_get_display_basename (doc);
 
     if (!name)
     {
-        g_critical ("%s: oops", G_STRLOC);
+        g_critical ("oops");
         name = "";
     }
 
     question = g_strdup_printf (_("Discard changes in file '%s'?"), name);
-    result = moo_edit_question_dialog (edit, question,
+    result = moo_edit_question_dialog (doc, question,
                                        _("If you reload the document, changes will be discarded"),
-                                       _("_Reload"));
+                                       _("_Reload"),
+                                       GTK_RESPONSE_CANCEL);
 
     g_free (question);
     return result;
 }
 
 gboolean
-_moo_edit_overwrite_modified_dialog (MooEdit    *edit)
+_moo_edit_overwrite_modified_dialog (MooEdit *doc)
 {
     const char *name;
     char *question, *secondary;
     gboolean result;
 
-    name = moo_edit_get_display_basename (edit);
+    name = moo_edit_get_display_basename (doc);
 
     if (!name)
     {
-        g_critical ("%s: oops", G_STRLOC);
+        g_critical ("oops");
         name = "";
     }
 
     question = g_strdup_printf (_("Overwrite modified file '%s'?"), name);
     secondary = g_strdup_printf (_("File '%s' was modified on disk by another process. If you save it, "
                                    "changes on disk will be lost."), name);
-    result = moo_edit_question_dialog (edit, question, secondary, _("Over_write"));
+    result = moo_edit_question_dialog (doc, question, secondary, _("Over_write"), GTK_RESPONSE_CANCEL);
 
     g_free (question);
     g_free (secondary);
@@ -665,22 +733,12 @@ _moo_text_regex_error_dialog (GtkWidget  *parent,
     if (error)
     {
         if (error->domain != G_REGEX_ERROR)
-        {
-            g_warning ("%s: unknown error domain", G_STRLOC);
-        }
-        else if (error->code != G_REGEX_ERROR_COMPILE &&
-                 error->code != G_REGEX_ERROR_OPTIMIZE &&
-                 error->code != G_REGEX_ERROR_REPLACE &&
-                 error->code != G_REGEX_ERROR_MATCH)
-        {
-            g_warning ("%s: unknown error code", G_STRLOC);
-        }
-
+            g_warning ("unknown error domain");
         msg_text = g_strdup (error->message);
     }
     else
     {
-        msg_text = g_strdup_printf ("Invalid regular expression");
+        msg_text = g_strdup_printf (_("Invalid regular expression"));
     }
 
     dialog = gtk_message_dialog_new (NULL,
