@@ -1,7 +1,7 @@
 /*
  *   mootextbuffer.h
  *
- *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
+ *   Copyright (C) 2004-2010 by Yevgen Muntyan <emuntyan@users.sourceforge.net>
  *
  *   This file is part of medit.  medit is free software; you can
  *   redistribute it and/or modify it under the terms of the
@@ -31,20 +31,23 @@ G_BEGIN_DECLS
 #define MOO_TEXT_BUFFER_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS ((obj), MOO_TYPE_TEXT_BUFFER, MooTextBufferClass))
 
 
-typedef struct _MooTextBufferPrivate  MooTextBufferPrivate;
-typedef struct _MooTextBufferClass    MooTextBufferClass;
+typedef struct MooTextBufferPrivate  MooTextBufferPrivate;
+typedef struct MooTextBufferClass    MooTextBufferClass;
 
-struct _MooTextBuffer
+struct MooTextBuffer
 {
     GtkTextBuffer  parent;
 
     MooTextBufferPrivate *priv;
 };
 
-struct _MooTextBufferClass
+struct MooTextBufferClass
 {
     GtkTextBufferClass parent_class;
 
+    gboolean (* undo)           (MooTextBuffer      *buffer);
+    gboolean (* redo)           (MooTextBuffer      *buffer);
+
     void (*cursor_moved)        (MooTextBuffer      *buffer,
                                  const GtkTextIter  *iter);
     void (*selection_changed)   (MooTextBuffer      *buffer);
@@ -80,12 +83,15 @@ gboolean    moo_text_buffer_get_highlight               (MooTextBuffer      *buf
 void        moo_text_buffer_set_brackets                (MooTextBuffer      *buffer,
                                                          const char         *brackets);
 
+gboolean    moo_text_buffer_can_redo                    (MooTextBuffer      *buffer);
+gboolean    moo_text_buffer_can_undo                    (MooTextBuffer      *buffer);
+void        moo_text_buffer_begin_non_undoable_action   (MooTextBuffer      *buffer);
+void        moo_text_buffer_end_non_undoable_action     (MooTextBuffer      *buffer);
+
 void        moo_text_buffer_freeze                      (MooTextBuffer      *buffer);
 void        moo_text_buffer_thaw                        (MooTextBuffer      *buffer);
 void        moo_text_buffer_begin_non_interactive_action(MooTextBuffer      *buffer);
 void        moo_text_buffer_end_non_interactive_action  (MooTextBuffer      *buffer);
-void        moo_text_buffer_begin_not_undoable_action   (MooTextBuffer      *buffer);
-void        moo_text_buffer_end_not_undoable_action     (MooTextBuffer      *buffer);
 
 gboolean    moo_text_buffer_has_text                    (MooTextBuffer      *buffer);
 gboolean    moo_text_buffer_has_selection               (MooTextBuffer      *buffer);
