@@ -1,17 +1,20 @@
 /*
  *   mootextsearch.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2010 by Yevgen Muntyan <emuntyan@users.sourceforge.net>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "mooedit/mootextsearch-private.h"
-#include "mooedit/gtksourceview/gtksourceview-api.h"
+#include "gtksourceview/gtksourceview-api.h"
 #include "mooutils/mooutils-misc.h"
 #include <glib/gregex.h>
 #include <string.h>
@@ -58,7 +61,7 @@ _moo_regex_new (GRegex *re)
 
     g_return_val_if_fail (re != NULL, NULL);
 
-    regex = moo_new (MooRegex);
+    regex = g_slice_new (MooRegex);
     regex->re = g_regex_ref (re);
     regex->ref_count_ = 1;
 
@@ -103,7 +106,7 @@ _moo_regex_unref (MooRegex *regex)
     if (!--regex->ref_count_)
     {
         g_regex_unref (regex->re);
-        moo_free (MooRegex, regex);
+        g_slice_free (MooRegex, regex);
     }
 }
 
@@ -454,14 +457,12 @@ moo_text_search_forward (const GtkTextIter      *start,
 
     regex = get_regex (str, flags, &error);
 
-    if (error)
-    {
-        g_warning ("%s: %s", G_STRLOC, error->message);
-        g_error_free (error);
-    }
-
     if (!regex)
+    {
+        g_warning ("%s", moo_error_message (error));
+        g_error_free (error);
         return FALSE;
+    }
 
     return _moo_text_search_regex_forward (start, end, regex,
                                            match_start, match_end,
@@ -511,14 +512,12 @@ moo_text_search_backward (const GtkTextIter      *start,
 
     regex = get_regex (str, flags, &error);
 
-    if (error)
-    {
-        g_warning ("%s: %s", G_STRLOC, error->message);
-        g_error_free (error);
-    }
-
     if (!regex)
+    {
+        g_warning ("%s", moo_error_message (error));
+        g_error_free (error);
         return FALSE;
+    }
 
     return _moo_text_search_regex_backward (start, end, regex,
                                             match_start, match_end,
@@ -559,7 +558,7 @@ moo_text_replace_regex_all_real (GtkTextIter            *start,
 
         if (!g_regex_check_replacement (replacement, &has_references, &error))
         {
-            g_warning ("%s: %s", G_STRLOC, error->message);
+            g_warning ("%s", moo_error_message (error));
             g_error_free (error);
             return 0;
         }
@@ -570,7 +569,7 @@ moo_text_replace_regex_all_real (GtkTextIter            *start,
 
             if (error)
             {
-                g_warning ("%s: %s", G_STRLOC, error->message);
+                g_warning ("%s", moo_error_message (error));
                 g_error_free (error);
                 return 0;
             }
@@ -649,7 +648,7 @@ moo_text_replace_regex_all_real (GtkTextIter            *start,
 
             if (!freeme_here)
             {
-                g_warning ("%s: %s", G_STRLOC, error->message);
+                g_warning ("%s", moo_error_message (error));
                 g_error_free (error);
                 g_match_info_free (match_info);
                 goto out;
@@ -783,14 +782,12 @@ moo_text_replace_all (GtkTextIter            *start,
         GError *error = NULL;
         MooRegex *regex = get_regex (text, flags, &error);
 
-        if (error)
-        {
-            g_warning ("%s: %s", G_STRLOC, error->message);
-            g_error_free (error);
-        }
-
         if (!regex)
+        {
+            g_warning ("%s", moo_error_message (error));
+            g_error_free (error);
             return 0;
+        }
 
         return _moo_text_replace_regex_all (start, end, regex, replacement,
                                             flags & MOO_TEXT_SEARCH_REPL_LITERAL);
@@ -861,14 +858,12 @@ _moo_text_replace_all_interactive (GtkTextIter            *start,
         GError *error = NULL;
         MooRegex *regex = get_regex (text, flags, &error);
 
-        if (error)
-        {
-            g_warning ("%s: %s", G_STRLOC, error->message);
-            g_error_free (error);
-        }
-
         if (!regex)
+        {
+            g_warning ("%s", moo_error_message (error));
+            g_error_free (error);
             return 0;
+        }
 
         return _moo_text_replace_regex_all_interactive (start, end, regex, replacement,
                                                         flags & MOO_TEXT_SEARCH_REPL_LITERAL,
