@@ -1,7 +1,7 @@
 /*
  *   moofile.c
  *
- *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
+ *   Copyright (C) 2004-2010 by Yevgen Muntyan <emuntyan@users.sourceforge.net>
  *
  *   This file is part of medit.  medit is free software; you can
  *   redistribute it and/or modify it under the terms of the
@@ -29,7 +29,7 @@
 #include "mooutils/mooutils-misc.h"
 #include "mooutils/mootype-macros.h"
 #include "mooutils/mooutils-debug.h"
-#include "mooutils/moocompat.h"
+#include "mooutils/moostat.h"
 #include "marshals.h"
 #include "mooutils/moo-mime.h"
 #include <glib/gstdio.h>
@@ -77,7 +77,7 @@ _moo_file_find_mime_type (MooFile    *file,
     if (!file->mime_type || !file->mime_type[0])
     {
         /* this should not happen */
-        _moo_message ("%s: oops, %s", G_STRLOC, file->display_name);
+        _moo_message ("oops, %s", file->display_name);
         file->mime_type = MOO_MIME_TYPE_UNKNOWN;
     }
 
@@ -169,7 +169,7 @@ moo_get_collation_key (const char *str,
       UCCreateCollator (NULL, 0, kUCCollateStandardOptions, &collator);
 
       if (!collator)
-        g_warning ("%s: UCCreateCollator failed", G_STRLOC);
+        g_warning ("UCCreateCollator failed");
     }
 
   return create_collation_key (collator, str, len);
@@ -194,7 +194,7 @@ moo_get_collation_key_for_filename (const char *str,
                         &collator);
 
       if (!collator)
-        g_warning ("%s: UCCreateCollator failed", G_STRLOC);
+        g_warning ("UCCreateCollator failed");
     }
 
   return create_collation_key (collator, str, len);
@@ -272,7 +272,7 @@ _moo_file_new (const char *dirname,
         g_return_val_if_fail (display_name != NULL, NULL);
     }
 
-    file = moo_new0 (MooFile);
+    file = g_slice_new0 (MooFile);
     file->ref_count = 1;
 
     file->name = g_strdup (basename);
@@ -312,8 +312,8 @@ _moo_file_unref (MooFile *file)
         g_free (file->case_display_name);
         g_free (file->collation_key);
         g_free (file->link_target);
-        moo_free (struct stat, file->statbuf);
-        moo_free (MooFile, file);
+        g_slice_free (struct stat, file->statbuf);
+        g_slice_free (MooFile, file);
     }
 }
 
@@ -337,16 +337,15 @@ _moo_file_stat (MooFile    *file,
     errno = 0;
 
     if (!file->statbuf)
-        file->statbuf = moo_new (struct stat);
+        file->statbuf = g_slice_new (struct stat);
 
-    if (g_lstat (fullname, file->statbuf) != 0)
+    if (moo_lstat (fullname, file->statbuf) != 0)
     {
         if (errno == ENOENT)
         {
             MOO_DEBUG_CODE({
                 gchar *display_name = g_filename_display_name (fullname);
-                _moo_message ("%s: file '%s' does not exist",
-                              G_STRLOC, display_name);
+                _moo_message ("file '%s' does not exist", display_name);
                 g_free (display_name);
             });
             file->info = 0;
@@ -356,9 +355,8 @@ _moo_file_stat (MooFile    *file,
             MOO_DEBUG_CODE({
                 int save_errno = errno;
                 gchar *display_name = g_filename_display_name (fullname);
-                _moo_message ("%s: error getting information for '%s': %s",
-                              G_STRLOC, display_name,
-                              g_strerror (save_errno));
+                _moo_message ("error getting information for '%s': %s",
+                              display_name, g_strerror (save_errno));
                 g_free (display_name);
             });
             file->info = MOO_FILE_INFO_IS_LOCKED | MOO_FILE_INFO_EXISTS;
@@ -376,14 +374,13 @@ _moo_file_stat (MooFile    *file,
             file->info |= MOO_FILE_INFO_IS_LINK;
             errno = 0;
 
-            if (g_stat (fullname, file->statbuf) != 0)
+            if (moo_stat (fullname, file->statbuf) != 0)
             {
                 if (errno == ENOENT)
                 {
                     MOO_DEBUG_CODE({
                         gchar *display_name = g_filename_display_name (fullname);
-                        _moo_message ("%s: file '%s' is a broken link",
-                                      G_STRLOC, display_name);
+                        _moo_message ("file '%s' is a broken link", display_name);
                         g_free (display_name);
                     });
                     file->info = MOO_FILE_INFO_IS_LINK;
@@ -393,9 +390,8 @@ _moo_file_stat (MooFile    *file,
                     MOO_DEBUG_CODE({
                         int save_errno = errno;
                         gchar *display_name = g_filename_display_name (fullname);
-                        _moo_message ("%s: error getting information for '%s': %s",
-                                      G_STRLOC, display_name,
-                                      g_strerror (save_errno));
+                        _moo_message ("error getting information for '%s': %s",
+                                      display_name, g_strerror (save_errno));
                         g_free (display_name);
                     });
                     file->info = MOO_FILE_INFO_IS_LOCKED | MOO_FILE_INFO_EXISTS;
@@ -410,9 +406,8 @@ _moo_file_stat (MooFile    *file,
                 MOO_DEBUG_CODE({
                     int save_errno = errno;
                     gchar *display_name = g_filename_display_name (fullname);
-                    _moo_message ("%s: error getting link target for '%s': %s",
-                                  G_STRLOC, display_name,
-                                  g_strerror (save_errno));
+                    _moo_message ("error getting link target for '%s': %s",
+                                  display_name, g_strerror (save_errno));
                     g_free (display_name);
                 });
             }
@@ -465,7 +460,7 @@ void
 _moo_file_free_statbuf (MooFile *file)
 {
     g_return_if_fail (file != NULL);
-    moo_free (struct stat, file->statbuf);
+    g_slice_free (struct stat, file->statbuf);
     file->statbuf = NULL;
     file->flags &= ~MOO_FILE_HAS_STAT;
 }
