@@ -1,13 +1,16 @@
 /*
  *   moofolder.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2010 by Yevgen Muntyan <emuntyan@users.sourceforge.net>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -257,7 +260,7 @@ _moo_folder_mem_usage (MooFolder *folder)
 MooFolder *
 _moo_folder_new_with_impl (MooFolderImpl *impl)
 {
-    MooFolder *folder = g_object_new (MOO_TYPE_FOLDER, NULL);
+    MooFolder *folder = MOO_FOLDER (g_object_new (MOO_TYPE_FOLDER, (const char*) NULL));
     folder->impl = impl;
     impl->proxy = folder;
     g_object_ref (impl->fs);
@@ -404,10 +407,10 @@ _moo_folder_set_wanted (MooFolder      *folder,
         folder->impl->populate_func (folder->impl))
     {
         folder->impl->populate_idle_id =
-                _moo_timeout_add_full (folder->impl->populate_priority,
-                                       folder->impl->populate_timeout,
-                                       folder->impl->populate_func,
-                                       folder->impl, NULL);
+                gdk_threads_add_timeout_full (folder->impl->populate_priority,
+                                              folder->impl->populate_timeout,
+                                              folder->impl->populate_func,
+                                              folder->impl, NULL);
     }
 
     g_object_unref (folder);
@@ -499,7 +502,7 @@ get_names (MooFolderImpl *impl)
         }
         else
         {
-            g_critical ("%s: _moo_file_new() failed for '%s'", G_STRLOC, name);
+            g_critical ("_moo_file_new() failed for '%s'", name);
         }
     }
 
@@ -612,19 +615,19 @@ get_stat_a_bit (MooFolderImpl *impl)
                 TIMER_CLEAR (impl->timer);
                 if (impl->populate_func (impl))
                     impl->populate_idle_id =
-                            _moo_timeout_add_full (impl->populate_priority,
-                                                   impl->populate_timeout,
-                                                   impl->populate_func,
-                                                   impl, NULL);
+                            gdk_threads_add_timeout_full (impl->populate_priority,
+                                                          impl->populate_timeout,
+                                                          impl->populate_func,
+                                                          impl, NULL);
             }
             else
             {
                 TIMER_CLEAR (impl->timer);
                 impl->populate_idle_id =
-                        _moo_timeout_add_full (impl->populate_priority,
-                                               impl->populate_timeout,
-                                               impl->populate_func,
-                                               impl, NULL);
+                        gdk_threads_add_timeout_full (impl->populate_priority,
+                                                      impl->populate_timeout,
+                                                      impl->populate_func,
+                                                      impl, NULL);
             }
         }
         else
@@ -834,8 +837,8 @@ start_monitor (MooFolderImpl *impl)
 
     if (!impl->fam_request)
     {
-        g_warning ("%s: moo_file_watch_create_monitor failed for path '%s'", G_STRLOC, impl->path);
-        g_warning ("%s: %s", G_STRLOC, error->message);
+        g_warning ("moo_file_watch_create_monitor failed for path '%s': %s",
+                   impl->path, moo_error_message (error));
         g_error_free (error);
         return;
     }
@@ -905,7 +908,7 @@ file_changed (MooFolderImpl *impl,
     if (!strcmp (name, impl->path))
     {
         if (!impl->reload_idle)
-            impl->reload_idle = moo_idle_add ((GSourceFunc) moo_folder_do_reload, impl);
+            impl->reload_idle = gdk_threads_add_idle ((GSourceFunc) moo_folder_do_reload, impl);
     }
 }
 
@@ -990,9 +993,7 @@ moo_folder_do_reload (MooFolderImpl *impl)
 
     if (!dir)
     {
-        g_warning ("%s: could not open directory %s",
-                   G_STRLOC, impl->path);
-        g_warning ("%s: %s", G_STRLOC, error->message);
+        g_warning ("could not open directory %s: %s", impl->path, moo_error_message (error));
         g_error_free (error);
         folder_emit_deleted (impl);
         return FALSE;
