@@ -1,13 +1,16 @@
 /*
  *   moofoldermodel.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2010 by Yevgen Muntyan <emuntyan@users.sourceforge.net>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #define MOO_FILE_VIEW_COMPILATION
@@ -103,7 +106,7 @@ moo_folder_model_class_init (MooFolderModelClass *klass)
                                              "folder",
                                              "folder",
                                              MOO_TYPE_FOLDER,
-                                             G_PARAM_READWRITE));
+                                             (GParamFlags) G_PARAM_READWRITE));
 }
 
 
@@ -362,12 +365,12 @@ G_STMT_START {                          \
     ITER_FILE(ip) = file;               \
 } G_STMT_END
 
-#ifdef MOO_DEBUG_ENABLED
+#ifdef MOO_DEBUG
 #if 0
 #define DEFINE_CHECK_ITER
 static void CHECK_ITER (MooFolderModel *model, GtkTreeIter *iter);
 #endif
-#endif /* MOO_DEBUG_ENABLED */
+#endif /* MOO_DEBUG */
 
 #ifndef DEFINE_CHECK_ITER
 #define CHECK_ITER(model,iter)
@@ -503,7 +506,7 @@ model_remove_moo_file (MooFile        *file,
 }
 
 
-#ifdef MOO_DEBUG_ENABLED
+#ifdef MOO_DEBUG
 #if 1
 #define CHECK_ORDER(order,n)                \
 G_STMT_START {                              \
@@ -518,7 +521,7 @@ G_STMT_START {                              \
     g_free (check);                         \
 } G_STMT_END
 #endif
-#endif /* MOO_DEBUG_ENABLED */
+#endif /* MOO_DEBUG */
 #ifndef CHECK_ORDER
 #define CHECK_ORDER(order,n)
 #endif
@@ -570,7 +573,7 @@ moo_folder_model_get_iter_impl (GtkTreeModel *tree_model,
 
     if (gtk_tree_path_get_depth (path) != 1)
     {
-        g_critical ("%s: oops", G_STRLOC);
+        g_critical ("oops");
         return FALSE;
     }
 
@@ -772,7 +775,8 @@ _moo_folder_model_new (MooFolder *folder)
 {
     g_return_val_if_fail (!folder || MOO_IS_FOLDER (folder), NULL);
     return GTK_TREE_MODEL (g_object_new (MOO_TYPE_FOLDER_MODEL,
-                                         "folder", folder, NULL));
+                                         "folder", folder,
+                                         (const char*) NULL));
 }
 
 
@@ -922,12 +926,13 @@ _moo_folder_filter_init (G_GNUC_UNUSED MooFolderFilter *filter)
 }
 
 
-GtkTreeModel*
+GtkTreeModel *
 _moo_folder_filter_new (MooFolderModel *model)
 {
     g_return_val_if_fail (MOO_IS_FOLDER_MODEL (model), NULL);
-    return g_object_new (MOO_TYPE_FOLDER_FILTER,
-                         "child-model", model, NULL);
+    return GTK_TREE_MODEL (g_object_new (MOO_TYPE_FOLDER_FILTER,
+                                         "child-model", model,
+                                         (const char*) NULL));
 }
 
 
