@@ -1,24 +1,30 @@
 /*
  *   mooaccel.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2010 by Yevgen Muntyan <emuntyan@users.sourceforge.net>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "mooutils/mooaccel.h"
 #include "mooutils/mooprefs.h"
+#include "mooutils/mooutils-tests.h"
+#include "mooutils/moocompat.h"
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 #include <string.h>
+#include <locale.h>
 
 #define MOO_ACCEL_PREFS_KEY "Shortcuts"
 
-#if GTK_CHECK_VERSION(2,10,0) && defined(GDK_WINDOWING_QUARTZ)
+#if defined(GDK_WINDOWING_QUARTZ)
 #define COMMAND_MASK GDK_META_MASK
 #else
 #define COMMAND_MASK GDK_CONTROL_MASK
@@ -37,9 +43,6 @@ static const char   *prefs_get_accel        (const char         *accel_path);
 static void          prefs_set_accel        (const char         *accel_path,
                                              const char         *accel);
 
-static gboolean      parse_accel            (const char         *accel,
-                                             guint              *key,
-                                             GdkModifierType    *mods);
 static void          my_gtk_accelerator_parse (const char       *accel,
                                              guint              *key,
                                              GdkModifierType    *mods);
@@ -96,7 +99,8 @@ accel_map_changed (G_GNUC_UNUSED GtkAccelMap *map,
 
 
 static char *
-accel_path_to_prefs_key (const char *accel_path)
+accel_path_to_prefs_key (const char *accel_path,
+                         gboolean    global)
 {
     if (accel_path && accel_path[0] == '<')
     {
@@ -108,7 +112,10 @@ accel_path_to_prefs_key (const char *accel_path)
     if (!accel_path || !accel_path[0])
         return NULL;
 
-    return g_strdup_printf (MOO_ACCEL_PREFS_KEY "/%s", accel_path);
+    if (global)
+        return g_strdup_printf (MOO_ACCEL_PREFS_KEY "/%s:global", accel_path);
+    else
+        return g_strdup_printf (MOO_ACCEL_PREFS_KEY "/%s", accel_path);
 }
 
 
@@ -120,7 +127,7 @@ prefs_set_accel (const char *accel_path,
 
     g_return_if_fail (accel != NULL);
 
-    key = accel_path_to_prefs_key (accel_path);
+    key = accel_path_to_prefs_key (accel_path, FALSE);
     g_return_if_fail (key != NULL);
 
     g_return_if_fail (moo_prefs_key_registered (key));
@@ -130,6 +137,50 @@ prefs_set_accel (const char *accel_path,
 }
 
 
+gboolean
+_moo_accel_prefs_get_global (const char *accel_path)
+{
+    char *key;
+    gboolean retval;
+
+    g_return_val_if_fail (accel_path != NULL, FALSE);
+
+    key = accel_path_to_prefs_key (accel_path, TRUE);
+    g_return_val_if_fail (key != NULL, FALSE);
+
+    /* XXX fix prefs for this */
+    moo_prefs_new_key_bool (key, FALSE);
+    retval = moo_prefs_get_bool (key);
+
+    g_free (key);
+    return retval;
+}
+
+void
+_moo_accel_prefs_set_global (const char *accel_path,
+                             gboolean    global)
+{
+    char *key;
+
+    g_return_if_fail (accel_path != NULL);
+
+    key = accel_path_to_prefs_key (accel_path, TRUE);
+    g_return_if_fail (key != NULL);
+
+    if (moo_prefs_key_registered (key))
+    {
+        moo_prefs_set_bool (key, global);
+    }
+    else if (global)
+    {
+        moo_prefs_new_key_bool (key, FALSE);
+        moo_prefs_set_bool (key, global);
+    }
+
+    g_free (key);
+}
+
+
 static void
 prefs_new_accel (const char *accel_path,
                  const char *default_accel)
@@ -138,7 +189,7 @@ prefs_new_accel (const char *accel_path,
 
     g_return_if_fail (accel_path != NULL && default_accel != NULL);
 
-    key = accel_path_to_prefs_key (accel_path);
+    key = accel_path_to_prefs_key (accel_path, FALSE);
     g_return_if_fail (key != NULL);
 
     if (!moo_prefs_key_registered (key))
@@ -154,7 +205,7 @@ prefs_get_accel (const char *accel_path)
     const char *accel;
     char *key;
 
-    key = accel_path_to_prefs_key (accel_path);
+    key = accel_path_to_prefs_key (accel_path, FALSE);
     g_return_val_if_fail (key != NULL, NULL);
 
     accel = moo_prefs_get_string (key);
@@ -170,7 +221,7 @@ set_accel (const char *accel_path,
            gboolean    set_gtk)
 {
     guint accel_key = 0;
-    GdkModifierType accel_mods = 0;
+    GdkModifierType accel_mods = (GdkModifierType) 0;
     const char *old_accel;
 
     g_return_if_fail (accel_path != NULL && accel != NULL);
@@ -229,7 +280,7 @@ get_accel (const char *accel_path)
 {
     g_return_val_if_fail (accel_path != NULL, NULL);
     init_accel_map ();
-    return g_hash_table_lookup (moo_accel_map, accel_path);
+    return (const char*) g_hash_table_lookup (moo_accel_map, accel_path);
 }
 
 
@@ -238,7 +289,7 @@ get_default_accel (const char *accel_path)
 {
     g_return_val_if_fail (accel_path != NULL, NULL);
     init_accel_map ();
-    return g_hash_table_lookup (moo_default_accel_map, accel_path);
+    return (const char*) g_hash_table_lookup (moo_default_accel_map, accel_path);
 }
 
 
@@ -284,7 +335,7 @@ _moo_accel_register (const char *accel_path,
     if (default_accel[0])
     {
         guint accel_key = 0;
-        GdkModifierType accel_mods = 0;
+        GdkModifierType accel_mods = (GdkModifierType) 0;
 
         my_gtk_accelerator_parse (default_accel, &accel_key, &accel_mods);
 
@@ -347,7 +398,7 @@ char *
 _moo_get_accel_label (const char *accel)
 {
     guint key = 0;
-    GdkModifierType mods = 0;
+    GdkModifierType mods = (GdkModifierType) 0;
 
     g_return_val_if_fail (accel != NULL, g_strdup (""));
 
@@ -375,7 +426,7 @@ moo_accel_translate_event (GtkWidget       *widget,
     if (keyval)
         *keyval = 0;
     if (mods)
-        *mods = 0;
+        *mods = (GdkModifierType) 0;
 
     if (widget && GTK_WIDGET_REALIZED (widget))
         keymap = gdk_keymap_get_for_display (gtk_widget_get_display (widget));
@@ -383,10 +434,10 @@ moo_accel_translate_event (GtkWidget       *widget,
         keymap = gdk_keymap_get_default ();
 
     gdk_keymap_translate_keyboard_state (keymap, event->hardware_keycode,
-                                         event->state, event->group,
+                                         (GdkModifierType) event->state, event->group,
                                          keyval, NULL, NULL, &consumed);
     if (mods)
-        *mods = event->state & ~consumed & MOO_ACCEL_MODS_MASK;
+        *mods = (GdkModifierType) (event->state & ~consumed & MOO_ACCEL_MODS_MASK);
 }
 
 gboolean
@@ -481,7 +532,7 @@ parse_key (const char *string)
 static GdkModifierType
 parse_mod (const char *string, gssize len)
 {
-    GdkModifierType mod = 0;
+    GdkModifierType mod = (GdkModifierType) 0;
     char *stripped;
 
     stripped = g_strstrip (g_ascii_strdown (string, len));
@@ -495,12 +546,10 @@ parse_mod (const char *string, gssize len)
     else if (!strcmp (stripped, "cmd") ||
              !strcmp (stripped, "command"))
         mod = COMMAND_MASK;
-#if GTK_CHECK_VERSION(2,10,0)
     else if (!strcmp (stripped, "meta"))
         mod = GDK_META_MASK;
-#endif
     else if (!strncmp (stripped, "mod", 3) &&
-              1 <= stripped[3] && stripped[3] <= 5 && !stripped[4])
+             '1' <= stripped[3] && stripped[3] <= '5' && !stripped[4])
     {
         switch (stripped[3])
         {
@@ -528,7 +577,7 @@ my_gtk_accelerator_parse (const char      *accel,
                           GdkModifierType *mods)
 {
     *key = 0;
-    *mods = 0;
+    *mods = (GdkModifierType) 0;
 
     while (*accel == '<')
     {
@@ -538,7 +587,7 @@ my_gtk_accelerator_parse (const char      *accel,
         if (!end || !(m = parse_mod (accel + 1, end - accel - 1)))
             return;
 
-        *mods |= m;
+        *mods = (GdkModifierType) (*mods | m);
         accel = end + 1;
     }
 
@@ -558,7 +607,7 @@ accel_parse_sep (const char     *accel,
 {
     char **pieces;
     guint n_pieces, i;
-    GdkModifierType mod = 0;
+    int mod = 0;
     guint key = 0;
 
     pieces = g_strsplit (accel, sep, 0);
@@ -586,19 +635,19 @@ out:
     if (keyval)
         *keyval = key;
     if (modifiers)
-        *modifiers = mod;
+        *modifiers = (GdkModifierType) mod;
     return key != 0;
 }
 
 
-static gboolean
-parse_accel (const char      *accel,
-             guint           *keyval,
-             GdkModifierType *modifiers)
+gboolean
+_moo_accel_parse (const char      *accel,
+                  guint           *keyval,
+                  GdkModifierType *modifiers)
 {
     guint key = 0;
     guint len;
-    GdkModifierType mods = 0;
+    GdkModifierType mods = (GdkModifierType) 0;
     char *p;
 
     g_return_val_if_fail (accel && accel[0], FALSE);
@@ -642,7 +691,7 @@ _moo_accel_normalize (const char *accel)
     if (!accel || !accel[0])
         return NULL;
 
-    if (parse_accel (accel, &key, &mods))
+    if (_moo_accel_parse (accel, &key, &mods))
     {
         return gtk_accelerator_name (key, mods);
     }
@@ -654,10 +703,7 @@ _moo_accel_normalize (const char *accel)
 }
 
 
-#ifdef MOO_ENABLE_UNIT_TESTS
-
-#include <mooutils/mooutils-tests.h>
-#include <locale.h>
+#define TEST_PREFS_KEY_PREFIX "FooBar/"
 
 static void
 test_moo_accel_register (void)
@@ -676,8 +722,8 @@ test_moo_accel_register (void)
     };
 
     PA bad_accels[] = {
-        { "<Something>/Test1/a", NULL, "" },
-        { "<Something>/Test1/b", "<dd", "" },
+        { "<Something>/Foobar/aa", NULL, "" },
+        { "<Something>/Foobar/bb", "<dd", "" },
     };
 
     PA cases[] = {
@@ -732,7 +778,7 @@ test_moo_accel_register (void)
         const char *second_accel = cases[i].second_accel;
         const char *third_accel = cases[i].third_accel;
         guint key = 0;
-        GdkModifierType mods = 0;
+        GdkModifierType mods = (GdkModifierType) 0;
 
         TEST_EXPECT_WARNING (0, "_moo_accel_register and friends for path %s", path);
 
@@ -890,16 +936,46 @@ test_moo_get_accel_label (void)
     setlocale (LC_ALL, "");
 }
 
+static void
+delete_prefs_keys (void)
+{
+    GSList *keys = moo_prefs_list_keys (MOO_PREFS_RC);
+    while (keys)
+    {
+        char *key = keys->data;
+
+        if (g_str_has_prefix (key, "Shortcuts/Foobar/"))
+            moo_prefs_delete_key (key);
+
+        g_free (key);
+        keys = g_slist_delete_link (keys, keys);
+    }
+}
+
+static gboolean
+test_suite_init (void)
+{
+    delete_prefs_keys ();
+    return TRUE;
+}
+
+static void
+test_suite_cleanup (void)
+{
+    delete_prefs_keys ();
+}
+
 void
 moo_test_mooaccel (void)
 {
-    MooTestSuite *suite = moo_test_suite_new ("mooutils/mooaccel.c", NULL, NULL, NULL);
-    moo_test_suite_add_test (suite, "test of _moo_get_accel_label()",
+    MooTestSuite *suite = moo_test_suite_new ("mooaccel", "mooutils/mooaccel.c",
+                                              (MooTestSuiteInit) test_suite_init,
+                                              (MooTestSuiteCleanup) test_suite_cleanup,
+                                              NULL);
+    moo_test_suite_add_test (suite, "_moo_get_accel_label", "test of _moo_get_accel_label()",
                              (MooTestFunc) test_moo_get_accel_label, NULL);
-    moo_test_suite_add_test (suite, "test of _moo_accel_normalize()",
+    moo_test_suite_add_test (suite, "_moo_accel_normalize", "test of _moo_accel_normalize()",
                              (MooTestFunc) test_moo_accel_normalize, NULL);
-    moo_test_suite_add_test (suite, "test of _moo_accel_register() and friends",
+    moo_test_suite_add_test (suite, "_moo_accel_register", "test of _moo_accel_register() and friends",
                              (MooTestFunc) test_moo_accel_register, NULL);
 }
-
-#endif
