@@ -1,15 +1,22 @@
 /*
  *   moobigpaned.c
  *
- *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
+ *   Copyright (C) 2004-2010 by Yevgen Muntyan <emuntyan@users.sourceforge.net>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+/**
+ * class:MooBigPaned: (parent GtkFrame) (constructable) (moo.private 1)
+ **/
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -22,12 +29,7 @@
 
 #ifdef MOO_COMPILATION
 #include "mooutils-misc.h"
-#else
-#if GLIB_CHECK_VERSION(2,10,0)
-#define MOO_OBJECT_REF_SINK(obj) g_object_ref_sink (obj)
-#else
-#define MOO_OBJECT_REF_SINK(obj) gtk_object_sink (g_object_ref (obj))
-#endif
+#include "moocompat.h"
 #endif
 
 typedef struct {
@@ -42,7 +44,13 @@ typedef struct {
     GHashTable *panes;
 } MooBigPanedConfig;
 
-struct MooBigPanedPrivate {
+typedef struct {
+    int        bbox_size;
+    GdkRegion *bbox_region;
+    GdkRegion *def_region;
+} DZ;
+
+struct _MooBigPanedPrivate {
     MooPanePosition order[4]; /* inner is paned[order[3]] */
     GtkWidget   *inner;
     GtkWidget   *outer;
@@ -55,11 +63,7 @@ struct MooBigPanedPrivate {
     GdkRectangle drop_rect;
     GdkRectangle drop_button_rect;
     GdkWindow   *drop_outline;
-    struct {
-        int        bbox_size;
-        GdkRegion *bbox_region;
-        GdkRegion *def_region;
-    } *dz;
+    DZ *dz;
     GdkRegion    *drop_region;
     guint         drop_region_is_buttons : 1;
 };
@@ -143,7 +147,7 @@ moo_big_paned_class_init (MooBigPanedClass *klass)
                                      g_param_spec_pointer ("pane-order",
                                              "pane-order",
                                              "pane-order",
-                                             G_PARAM_READWRITE));
+                                             (GParamFlags) G_PARAM_READWRITE));
 
     g_object_class_install_property (gobject_class,
                                      PROP_ENABLE_HANDLE_DRAG,
@@ -151,7 +155,7 @@ moo_big_paned_class_init (MooBigPanedClass *klass)
                                              "enable-handle-drag",
                                              "enable-handle-drag",
                                              TRUE,
-                                             G_PARAM_CONSTRUCT | G_PARAM_WRITABLE));
+                                             (GParamFlags) (G_PARAM_CONSTRUCT | G_PARAM_WRITABLE)));
 
     g_object_class_install_property (gobject_class,
                                      PROP_ENABLE_DETACHING,
@@ -159,7 +163,7 @@ moo_big_paned_class_init (MooBigPanedClass *klass)
                                              "enable-detaching",
                                              "enable-detaching",
                                              FALSE,
-                                             G_PARAM_CONSTRUCT | G_PARAM_WRITABLE));
+                                             (GParamFlags) (G_PARAM_CONSTRUCT | G_PARAM_WRITABLE)));
 
     g_object_class_install_property (gobject_class,
                                      PROP_HANDLE_CURSOR_TYPE,
@@ -168,7 +172,7 @@ moo_big_paned_class_init (MooBigPanedClass *klass)
                                              "handle-cursor-type",
                                              GDK_TYPE_CURSOR_TYPE,
                                              GDK_HAND2,
-                                             G_PARAM_CONSTRUCT | G_PARAM_READWRITE));
+                                             (GParamFlags) (G_PARAM_CONSTRUCT | G_PARAM_READWRITE)));
 
     signals[CONFIG_CHANGED] =
             g_signal_new ("config-changed",
@@ -204,11 +208,11 @@ moo_big_paned_init (MooBigPaned *paned)
         GtkWidget *child;
 
         paned->paned[i] = child =
-                g_object_new (MOO_TYPE_PANED,
-                              "pane-position", (MooPanePosition) i,
-                              NULL);
+                GTK_WIDGET (g_object_new (MOO_TYPE_PANED,
+                                          "pane-position", (MooPanePosition) i,
+                                          (const char*) NULL));
 
-        MOO_OBJECT_REF_SINK (child);
+        g_object_ref_sink (child);
         gtk_widget_show (child);
 
         g_signal_connect_after (child, "set-pane-size",
@@ -268,7 +272,7 @@ void
 moo_big_paned_set_pane_order (MooBigPaned *paned,
                               int         *order)
 {
-    MooPanePosition new_order[4] = {8, 8, 8, 8};
+    MooPanePosition new_order[4];
     int i;
     GtkWidget *child;
 
@@ -277,9 +281,8 @@ moo_big_paned_set_pane_order (MooBigPaned *paned,
 
     for (i = 0; i < 4; ++i)
     {
-        g_return_if_fail (new_order[i] >= 4);
         g_return_if_fail (0 <= order[i] && order[i] < 4);
-        new_order[i] = order[i];
+        new_order[i] = (MooPanePosition) order[i];
     }
 
     g_return_if_fail (check_children_order (paned));
@@ -340,7 +343,7 @@ moo_big_paned_finalize (GObject *object)
 
     if (paned->priv->drop_outline)
     {
-        g_critical ("%s: oops", G_STRLOC);
+        g_critical ("oops");
         gdk_window_set_user_data (paned->priv->drop_outline, NULL);
         gdk_window_destroy (paned->priv->drop_outline);
     }
@@ -349,10 +352,10 @@ moo_big_paned_finalize (GObject *object)
 }
 
 
-GtkWidget*
+GtkWidget *
 moo_big_paned_new (void)
 {
-    return g_object_new (MOO_TYPE_BIG_PANED, NULL);
+    return GTK_WIDGET (g_object_new (MOO_TYPE_BIG_PANED, (const char*) NULL));
 }
 
 
@@ -455,7 +458,7 @@ get_pane_config (MooBigPaned     *paned,
 
     g_return_val_if_fail (id != NULL, NULL);
 
-    if (!(params = g_hash_table_lookup (paned->priv->config->panes, id)))
+    if (!(params = (MooPaneParams*) g_hash_table_lookup (paned->priv->config->panes, id)))
         return NULL;
 
     for (pos = 0; pos < 4 && !found; ++pos)
@@ -467,10 +470,10 @@ get_pane_config (MooBigPaned     *paned,
              link != NULL && !found;
              link = link->next)
         {
-            if (strcmp (link->data, id) == 0)
+            if (strcmp ((const char*) link->data, id) == 0)
             {
                 *index = index_here;
-                *position = pos;
+                *position = (MooPanePosition) pos;
                 found = TRUE;
             }
             else if (g_hash_table_lookup (paned->priv->panes, link->data))
@@ -783,7 +786,7 @@ moo_big_paned_lookup_pane (MooBigPaned *paned,
 {
     g_return_val_if_fail (MOO_IS_BIG_PANED (paned), NULL);
     g_return_val_if_fail (pane_id != NULL, NULL);
-    return g_hash_table_lookup (paned->priv->panes, pane_id);
+    return MOO_PANE (g_hash_table_lookup (paned->priv->panes, pane_id));
 }
 
 
@@ -879,7 +882,7 @@ moo_big_paned_set_property (GObject      *object,
     switch (prop_id)
     {
         case PROP_PANE_ORDER:
-            moo_big_paned_set_pane_order (paned, g_value_get_pointer (value));
+            moo_big_paned_set_pane_order (paned, (int*) g_value_get_pointer (value));
             break;
 
         case PROP_ENABLE_HANDLE_DRAG:
@@ -1014,7 +1017,7 @@ get_drop_zones (MooBigPaned *paned)
 
     g_return_if_fail (paned->priv->dz == NULL);
 
-    paned->priv->dz = g_malloc0 (4 * sizeof *paned->priv->dz);
+    paned->priv->dz = g_new0 (DZ, 4);
 
     for (pos = 0; pos < 4; ++pos)
     {
@@ -1247,7 +1250,8 @@ handle_drag_motion (MooPaned       *child,
 
     if (paned->priv->drop_pos >= 0)
     {
-        get_drop_area (paned, child, paned->priv->drop_pos,
+        get_drop_area (paned, child,
+                       (MooPanePosition) paned->priv->drop_pos,
                        paned->priv->drop_button_index,
                        &paned->priv->drop_rect,
                        &paned->priv->drop_button_rect);
@@ -1312,7 +1316,7 @@ handle_drag_end (MooPaned    *child,
         return;
     }
 
-    new_pos = paned->priv->drop_pos;
+    new_pos = (MooPanePosition) paned->priv->drop_pos;
     new_index = paned->priv->drop_button_index;
     cleanup_drag (paned);
 
@@ -1715,7 +1719,7 @@ paned_config_serialize (MooPanedConfig *config,
 
     for (l = config->order; l != NULL; l = l->next)
     {
-        char *id = l->data;
+        char *id = (char*) l->data;
         g_string_append_printf (string, ",%s", id);
     }
 }
