@@ -1,21 +1,29 @@
 /*
  *   mooentry.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2010 by Yevgen Muntyan <emuntyan@users.sourceforge.net>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+/**
+ * class:MooEntry: (parent GtkEntry) (constructable) (moo.private 1)
+ **/
+
 #include "marshals.h"
 #include "mooutils/mooaccel.h"
 #include "mooutils/mooentry.h"
 #include "mooutils/mooundo.h"
 #include "mooutils/mooutils-gobject.h"
 #include "mooutils/mooeditops.h"
+#include "mooutils/mooi18n.h"
 #include <gtk/gtkbindings.h>
 #include <gtk/gtkimagemenuitem.h>
 #include <gtk/gtkseparatormenuitem.h>
@@ -84,11 +92,11 @@ static gboolean moo_entry_get_selection_bounds (GtkEditable     *editable,
 static void     moo_entry_changed           (GtkEditable        *editable);
 
 static void     init_undo_actions           (void);
-static gpointer insert_action_new           (GtkEditable        *editable,
+static MooUndoAction *insert_action_new     (GtkEditable        *editable,
                                              const gchar        *text,
                                              gint                length,
                                              gint               *position);
-static gpointer delete_action_new           (GtkEditable        *editable,
+static MooUndoAction *delete_action_new     (GtkEditable        *editable,
                                              gint                start_pos,
                                              gint                end_pos);
 
@@ -122,7 +130,7 @@ moo_entry_get_type (void)
             (GInterfaceInitFunc) moo_entry_undo_ops_init, NULL, NULL
         };
 
-        type = g_type_register_static (GTK_TYPE_ENTRY, "MooEntry", &info, 0);
+        type = g_type_register_static (GTK_TYPE_ENTRY, "MooEntry", &info, (GTypeFlags) 0);
         g_type_add_interface_static (type, GTK_TYPE_EDITABLE, &editable_info);
         g_type_add_interface_static (type, MOO_TYPE_UNDO_OPS, &undo_ops_info);
     }
@@ -186,7 +194,7 @@ moo_entry_class_init (MooEntryClass *klass)
                                              "enable-undo",
                                              "enable-undo",
                                              TRUE,
-                                             G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+                                             (GParamFlags) (G_PARAM_READWRITE | G_PARAM_CONSTRUCT)));
 
     g_object_class_install_property (gobject_class,
                                      PROP_ENABLE_UNDO_MENU,
@@ -194,7 +202,7 @@ moo_entry_class_init (MooEntryClass *klass)
                                              "enable-undo-menu",
                                              "enable-undo-menu",
                                              TRUE,
-                                             G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+                                             (GParamFlags) (G_PARAM_READWRITE | G_PARAM_CONSTRUCT)));
 
     g_object_class_install_property (gobject_class,
                                      PROP_GRAB_SELECTION,
@@ -202,7 +210,7 @@ moo_entry_class_init (MooEntryClass *klass)
                                              "grab-selection",
                                              "grab-selection",
                                              TRUE,
-                                             G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+                                             (GParamFlags) (G_PARAM_READWRITE | G_PARAM_CONSTRUCT)));
 
     g_object_class_install_property (gobject_class,
                                      PROP_EMPTY,
@@ -218,7 +226,7 @@ moo_entry_class_init (MooEntryClass *klass)
                                              "use-special-chars-menu",
                                              "use-special-chars-menu",
                                              FALSE,
-                                             G_PARAM_READWRITE));
+                                             (GParamFlags) G_PARAM_READWRITE));
 
     signals[UNDO] = g_signal_lookup ("undo", GTK_TYPE_ENTRY);
 
@@ -226,7 +234,7 @@ moo_entry_class_init (MooEntryClass *klass)
         signals[UNDO] =
                 g_signal_new ("undo",
                               G_OBJECT_CLASS_TYPE (klass),
-                              G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+                              (GSignalFlags) (G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION),
                               G_STRUCT_OFFSET (MooEntryClass, undo),
                               NULL, NULL,
                               _moo_marshal_VOID__VOID,
@@ -238,7 +246,7 @@ moo_entry_class_init (MooEntryClass *klass)
         signals[REDO] =
                 g_signal_new ("redo",
                               G_OBJECT_CLASS_TYPE (klass),
-                              G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+                              (GSignalFlags) (G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION),
                               G_STRUCT_OFFSET (MooEntryClass, redo),
                               NULL, NULL,
                               _moo_marshal_VOID__VOID,
@@ -247,7 +255,7 @@ moo_entry_class_init (MooEntryClass *klass)
     signals[DELETE_TO_START] =
             _moo_signal_new_cb ("delete-to-start",
                                 G_OBJECT_CLASS_TYPE (klass),
-                                G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+                                (GSignalFlags) (G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION),
                                 G_CALLBACK (moo_entry_delete_to_start),
                                 NULL, NULL,
                                 _moo_marshal_VOID__VOID,
@@ -258,7 +266,7 @@ moo_entry_class_init (MooEntryClass *klass)
                                   MOO_ACCEL_CTRL_MASK,
                                   "undo", 0);
     gtk_binding_entry_add_signal (binding_set, GDK_z,
-                                  MOO_ACCEL_CTRL_MASK | GDK_SHIFT_MASK,
+                                  (GdkModifierType) (MOO_ACCEL_CTRL_MASK | GDK_SHIFT_MASK),
                                   "redo", 0);
     gtk_binding_entry_add_signal (binding_set, GDK_u,
                                   GDK_CONTROL_MASK,
@@ -462,10 +470,10 @@ moo_entry_undo_ops_init (MooUndoOpsIface *iface)
 }
 
 
-GtkWidget*
+GtkWidget *
 moo_entry_new (void)
 {
-    return g_object_new (MOO_TYPE_ENTRY, NULL);
+    return GTK_WIDGET (g_object_new (MOO_TYPE_ENTRY, (const char*) NULL));
 }
 
 
@@ -497,7 +505,7 @@ special_char_item_activated (GtkWidget *item,
 {
     const char *text;
 
-    text = g_object_get_data (G_OBJECT (item), "moo-entry-special-char");
+    text = (const char*) g_object_get_data (G_OBJECT (item), "moo-entry-special-char");
     g_return_if_fail (text != NULL);
 
     moo_entry_insert_at_cursor (entry, text, -1);
@@ -524,8 +532,10 @@ static GtkWidget *
 create_special_chars_menu (MooEntry *entry)
 {
     GtkWidget *menu = gtk_menu_new ();
-    create_special_char_item (entry, menu, "Line End", "\n");
-    create_special_char_item (entry, menu, "Tab", "\t");
+    /* menu item in Insert Special Character submenu of text entry context menu */
+    create_special_char_item (entry, menu, C_("insert-special-char", "Line End"), "\n");
+    /* menu item in Insert Special Character submenu of text entry context menu */
+    create_special_char_item (entry, menu, C_("insert-special-char", "Tab"), "\t");
     gtk_widget_show_all (menu);
     return menu;
 }
@@ -567,7 +577,8 @@ moo_entry_populate_popup (GtkEntry           *gtkentry,
         gtk_widget_show (item);
         gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 
-        item = gtk_menu_item_new_with_label ("Insert Special Character");
+        /* label of Insert Special Character submenu in text entry context menu */
+        item = gtk_menu_item_new_with_label (C_("insert-special-char", "Insert Special Character"));
         gtk_widget_show (item);
         gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 
@@ -629,8 +640,8 @@ moo_entry_do_insert_text (GtkEditable        *editable,
     if (length > 0)
     {
         moo_undo_stack_add_action (MOO_ENTRY(editable)->priv->undo_stack,
-                                 INSERT_ACTION_TYPE,
-                                 insert_action_new (editable, text, length, position));
+                                   INSERT_ACTION_TYPE,
+                                   insert_action_new (editable, text, length, position));
         parent_editable_iface->do_insert_text (editable, text, length, position);
         moo_undo_ops_can_undo_changed (G_OBJECT (editable));
     }
@@ -782,7 +793,7 @@ init_undo_actions (void)
 }
 
 
-static gpointer
+static MooUndoAction *
 insert_action_new (G_GNUC_UNUSED GtkEditable *editable,
                    const gchar        *text,
                    gint                length,
@@ -802,11 +813,11 @@ insert_action_new (G_GNUC_UNUSED GtkEditable *editable,
     action->length = length;
     action->chars = g_utf8_strlen (text, length);
 
-    return action;
+    return (MooUndoAction*) action;
 }
 
 
-static gpointer
+static MooUndoAction *
 delete_action_new (GtkEditable        *editable,
                    gint                start_pos,
                    gint                end_pos)
@@ -828,7 +839,7 @@ delete_action_new (GtkEditable        *editable,
     else
         action->forward = FALSE;
 
-    return action;
+    return (MooUndoAction*) action;
 }
 
 
