@@ -1,18 +1,26 @@
 /*
  *   moohistorycombo.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2010 by Yevgen Muntyan <emuntyan@users.sourceforge.net>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+/**
+ * class:MooHistoryCombo: (parent MooCombo) (constructable) (moo.private 1)
+ **/
+
 #include "marshals.h"
 #include "mooutils/moohistorycombo.h"
 #include "mooutils/mooutils-misc.h"
+#include "mooutils/moocompat.h"
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 #include <string.h>
@@ -107,7 +115,7 @@ moo_history_combo_class_init (MooHistoryComboClass *klass)
                                              "list",
                                              "list",
                                              MOO_TYPE_HISTORY_LIST,
-                                             G_PARAM_READWRITE));
+                                             (GParamFlags) G_PARAM_READWRITE));
 
     g_object_class_install_property (gobject_class,
                                      PROP_SORT_HISTORY,
@@ -115,7 +123,7 @@ moo_history_combo_class_init (MooHistoryComboClass *klass)
                                              "sort-history",
                                              "sort-history",
                                              TRUE,
-                                             G_PARAM_READWRITE));
+                                             (GParamFlags) G_PARAM_READWRITE));
 
     g_object_class_install_property (gobject_class,
                                      PROP_SORT_COMPLETION,
@@ -123,7 +131,7 @@ moo_history_combo_class_init (MooHistoryComboClass *klass)
                                              "sort-completion",
                                              "sort-completion",
                                              FALSE,
-                                             G_PARAM_READWRITE));
+                                             (GParamFlags) G_PARAM_READWRITE));
 
     g_object_class_install_property (gobject_class,
                                      PROP_ENABLE_COMPLETION,
@@ -131,7 +139,7 @@ moo_history_combo_class_init (MooHistoryComboClass *klass)
                                              "enable-completion",
                                              "enable-completion",
                                              TRUE,
-                                             G_PARAM_READWRITE));
+                                             (GParamFlags) G_PARAM_READWRITE));
 
     g_object_class_install_property (gobject_class,
                                      PROP_HISTORY_LIST_ID,
@@ -198,7 +206,7 @@ moo_history_combo_set_property (GObject        *object,
     switch (prop_id)
     {
         case PROP_LIST:
-            moo_history_combo_set_list (combo, g_value_get_object (value));
+            moo_history_combo_set_list (combo, MOO_HISTORY_LIST (g_value_get_object (value)));
             break;
 
         case PROP_SORT_HISTORY:
@@ -342,7 +350,7 @@ default_sort_func (GtkTreeModel       *model,
                    GtkTreeIter        *a,
                    GtkTreeIter        *b)
 {
-    MooHistoryItem *e1 = NULL, *e2 = NULL;
+    MooHistoryListItem *e1 = NULL, *e2 = NULL;
     int result;
 
     /* XXX 0 is not good */
@@ -360,8 +368,8 @@ default_sort_func (GtkTreeModel       *model,
     else
         result = strcmp (e1->data, e2->data);
 
-    moo_history_item_free (e1);
-    moo_history_item_free (e2);
+    moo_history_list_item_free (e1);
+    moo_history_list_item_free (e2);
     return result;
 }
 
@@ -419,9 +427,9 @@ moo_history_combo_changed (MooCombo *combo)
         if (hist_combo->priv->completion_popup_timeout_id)
             g_source_remove (hist_combo->priv->completion_popup_timeout_id);
         hist_combo->priv->completion_popup_timeout_id =
-                _moo_timeout_add (hist_combo->priv->completion_popup_timeout,
-                                  (GSourceFunc) popup,
-                                  combo);
+                gdk_threads_add_timeout (hist_combo->priv->completion_popup_timeout,
+                                         (GSourceFunc) popup,
+                                         combo);
     }
     else
     {
@@ -450,7 +458,7 @@ default_filter_func (const char         *entry_text,
                      G_GNUC_UNUSED gpointer data)
 {
     gboolean visible;
-    MooHistoryItem *e = NULL;
+    MooHistoryListItem *e = NULL;
 
     gtk_tree_model_get (model, iter, 0, &e, -1);
 
@@ -466,7 +474,7 @@ default_filter_func (const char         *entry_text,
         visible = TRUE;
     }
 
-    moo_history_item_free (e);
+    moo_history_list_item_free (e);
     return visible ? TRUE : FALSE;
 }
 
@@ -476,7 +484,7 @@ moo_history_combo_new (const char *user_id)
 {
     MooHistoryCombo *combo;
 
-    combo = g_object_new (MOO_TYPE_HISTORY_COMBO, NULL);
+    combo = MOO_HISTORY_COMBO (g_object_new (MOO_TYPE_HISTORY_COMBO, (const char*) NULL));
 
     if (user_id)
     {
@@ -503,7 +511,7 @@ moo_history_combo_commit (MooHistoryCombo *combo)
 {
     GtkTreeIter iter;
     const char *text;
-    MooHistoryItem *freeme = NULL;
+    MooHistoryListItem *freeme = NULL;
 
     g_return_if_fail (MOO_IS_HISTORY_COMBO (combo));
 
@@ -521,7 +529,7 @@ moo_history_combo_commit (MooHistoryCombo *combo)
     moo_history_combo_add_text (combo, text);
 
     if (freeme)
-        moo_history_item_free (freeme);
+        moo_history_list_item_free (freeme);
 }
 
 
@@ -576,14 +584,14 @@ cell_data_func (G_GNUC_UNUSED GtkCellLayout *cell_layout,
                 GtkTreeModel       *model,
                 GtkTreeIter        *iter)
 {
-    MooHistoryItem *e = NULL;
+    MooHistoryListItem *e = NULL;
 
     gtk_tree_model_get (model, iter, 0, &e, -1);
 
     if (e)
     {
         g_object_set (cell, "text", e->display, NULL);
-        moo_history_item_free (e);
+        moo_history_list_item_free (e);
     }
 }
 
@@ -594,10 +602,10 @@ row_separator_func (GtkTreeModel       *model,
                     G_GNUC_UNUSED gpointer data)
 {
     gboolean separator;
-    MooHistoryItem *item = NULL;
+    MooHistoryListItem *item = NULL;
     gtk_tree_model_get (model, iter, 0, &item, -1);
     separator = item == NULL;
-    moo_history_item_free (item);
+    moo_history_list_item_free (item);
     return separator;
 }
 
@@ -607,7 +615,7 @@ get_text_func (GtkTreeModel       *model,
                GtkTreeIter        *iter,
                G_GNUC_UNUSED gpointer data)
 {
-    MooHistoryItem *e = NULL;
+    MooHistoryListItem *e = NULL;
     char *text;
 
     gtk_tree_model_get (model, iter, 0, &e, -1);
@@ -616,6 +624,6 @@ get_text_func (GtkTreeModel       *model,
     g_return_val_if_fail (g_utf8_validate (e->data, -1, NULL), NULL);
 
     text = g_strdup (e->data);
-    moo_history_item_free (e);
+    moo_history_list_item_free (e);
     return text;
 }
