@@ -1,7 +1,7 @@
 /*
  *   mooutils-fs.c
  *
- *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
+ *   Copyright (C) 2004-2010 by Yevgen Muntyan <emuntyan@users.sourceforge.net>
  *
  *   This file is part of medit.  medit is free software; you can
  *   redistribute it and/or modify it under the terms of the
@@ -18,12 +18,12 @@
 #endif
 
 #include "mooutils/mooutils-fs.h"
-// #include "mooutils/mooutils-file.h"
+#include "mooutils/moostat.h"
 #include "mooutils/mooutils-debug.h"
 #include "mooutils/mooutils-mem.h"
 #include "mooutils/mootype-macros.h"
-#include "mooutils/moocompat.h"
 #include "mooutils/mooi18n.h"
+#include <mooutils/mooutils-tests.h>
 
 #include <stdio.h>
 #include <string.h>
@@ -243,7 +243,7 @@ rm_fr (const char *path,
                      /* This is error message in file selector when rm
                         fails to delete a file or folder */
                      _("'rm' command failed: %s"),
-                     child_err ? child_err : "");
+                     MOO_NZS (child_err));
 
         g_free (child_err);
         return FALSE;
@@ -376,7 +376,7 @@ _moo_create_dir (const char *path,
 
     errno = 0;
 
-    if (g_stat (path, &buf) != 0 && errno != ENOENT)
+    if (moo_stat (path, &buf) != 0 && errno != ENOENT)
     {
         int err_code = errno;
         utf8_path = g_filename_display_name (path);
@@ -505,7 +505,7 @@ moo_filenames_from_locale (char **files)
         conv[i] = moo_filename_from_locale (*files);
 
         if (!conv[i])
-            g_warning ("%s: could not convert '%s' to UTF8", G_STRLOC, *files);
+            g_warning ("could not convert '%s' to UTF8", *files);
         else
             ++i;
     }
@@ -597,7 +597,7 @@ _moo_chdir (const char *path)
 //     if (_moo_chdir (dirname) != 0)
 //     {
 //         int err = errno;
-//         g_warning ("%s: %s", G_STRLOC, g_strerror (err));
+//         g_warning ("%s", g_strerror (err));
 //     }
 //     else
 //     {
@@ -610,7 +610,7 @@ _moo_chdir (const char *path)
 //         if (_moo_chdir (working_dir) != 0)
 //         {
 //             int err = errno;
-//             g_warning ("%s: %s", G_STRLOC, g_strerror (err));
+//             g_warning ("%s", g_strerror (err));
 //         }
 //     }
 //
@@ -734,8 +734,6 @@ normalize_full_path_unix (const char *path)
 
 #endif /* !__WIN32__ */
 
-#if defined(__WIN32__) || defined(MOO_ENABLE_UNIT_TESTS)
-
 static char *
 normalize_full_path_win32 (const char *fullpath)
 {
@@ -844,8 +842,6 @@ normalize_full_path_win32 (const char *fullpath)
     return path;
 }
 
-#endif
-
 static char *
 normalize_path (const char *filename)
 {
@@ -898,10 +894,6 @@ _moo_path_is_absolute (const char *path)
 }
 
 
-#ifdef MOO_ENABLE_UNIT_TESTS
-
-#include <mooutils/mooutils-tests.h>
-
 static void
 test_normalize_path_one (const char *path,
                          const char *expected,
@@ -1134,18 +1126,16 @@ moo_test_mooutils_fs (void)
 {
     MooTestSuite *suite;
 
-    suite = moo_test_suite_new ("mooutils/mooutils-fs.c", NULL, NULL, NULL);
+    suite = moo_test_suite_new ("mooutils-fs", "mooutils/mooutils-fs.c", NULL, NULL, NULL);
 
-    moo_test_suite_add_test (suite, "test of _moo_normalize_file_path()",
+    moo_test_suite_add_test (suite, "_moo_normalize_file_path", "test of _moo_normalize_file_path()",
                              (MooTestFunc) test_normalize_file_path, NULL);
 #ifndef __WIN32__
-    moo_test_suite_add_test (suite, "test of normalize_full_path_win32()",
+    moo_test_suite_add_test (suite, "normalize_full_path_win32", "test of normalize_full_path_win32()",
                              (MooTestFunc) test_normalize_file_path_win32, NULL);
 #endif
 }
 
-#endif /* MOO_ENABLE_UNIT_TESTS */
-
 
 /**********************************************************************/
 /* MSLU for poor
@@ -1458,14 +1448,12 @@ _moo_glob_new (const char *pattern)
 
     g_free (re_pattern);
 
-    if (error)
-    {
-        g_warning ("%s: %s", G_STRLOC, error->message);
-        g_error_free (error);
-    }
-
     if (!re)
+    {
+        g_warning ("%s", moo_error_message (error));
+        g_error_free (error);
         return NULL;
+    }
 
     gl = g_new0 (MooGlob, 1);
     gl->re = re;
