@@ -1,7 +1,7 @@
 /*
  *   mooutils-misc.c
  *
- *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
+ *   Copyright (C) 2004-2010 by Yevgen Muntyan <emuntyan@users.sourceforge.net>
  *
  *   This file is part of medit.  medit is free software; you can
  *   redistribute it and/or modify it under the terms of the
@@ -13,16 +13,21 @@
  *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "config-dirs.h"
+#include "config.h"
 #include "mooutils/mooutils-misc.h"
 #include "mooutils/mooutils-fs.h"
-#include "mooutils/mooutils-file.h"
+#include "mooutils/moofilewriter.h"
 #include "mooutils/mooutils-debug.h"
 #include "mooutils/mooi18n.h"
 #include "mooutils/mooatom.h"
 #include "mooutils/mooonce.h"
 #include "mooutils/mooutils-misc.h"
-#include "glade/moologwindow-gxml.h"
+#include "mooutils/mooutils-enums.h"
+#include <mooutils/mooutils-tests.h>
+#include "mooutils/moocompat.h"
+#include "mooutils/mootype-macros.h"
+#include "mooutils/mooarray.h"
+#include "mooutils/moologwindow-gxml.h"
 #include <gtk/gtk.h>
 #include <glib/gmappedfile.h>
 #include <glib/gstdio.h>
@@ -41,7 +46,7 @@
 #include <gdk/gdkx.h>
 #endif
 
-#if defined(GDK_WINDOWING_QUARTZ)
+#ifdef MOO_USE_QUARTZ
 #include <gdk/gdkquartz.h>
 #endif
 
@@ -50,16 +55,13 @@
 #include <shellapi.h>
 #endif
 
+MOO_DEFINE_OBJECT_ARRAY_FULL (MooObjectArray, moo_object_array, GObject)
+MOO_DEFINE_QUARK (moo-error, moo_error_quark)
 
 G_LOCK_DEFINE_STATIC (moo_user_data_dir);
 static char *moo_app_instance_name;
+static char *moo_display_app_name;
 static char *moo_user_data_dir;
-static char *moo_temp_dir;
-
-static void         moo_install_atexit      (void);
-static void         moo_remove_tempdir      (void);
-static const char  *moo_get_prgname         (void);
-
 
 #ifdef __WIN32__
 
@@ -87,7 +89,7 @@ _moo_find_script (const char *name,
     {
         char **dirs, **p;
 
-        dirs = moo_get_data_subdirs ("scripts", MOO_DATA_SHARE, NULL);
+        dirs = moo_get_data_subdirs ("scripts");
 
         for (p = dirs; p && *p; ++p)
         {
@@ -105,7 +107,7 @@ _moo_find_script (const char *name,
 
     if (!path)
     {
-        g_warning ("%s: could not find %s script", G_STRLOC, name);
+        g_warning ("could not find %s script", name);
         path = g_strdup (name);
     }
 
@@ -140,20 +142,20 @@ open_uri (const char *uri,
 {
     gboolean result = FALSE;
     const char *argv[3];
-    GError *err = NULL;
+    GError *error = NULL;
 
     argv[0] = get_xdg_script (email ? SCRIPT_XDG_EMAIL : SCRIPT_XDG_OPEN);
     argv[1] = uri;
     argv[2] = NULL;
 
     result = g_spawn_async (NULL, (char**) argv, NULL, (GSpawnFlags) 0, NULL, NULL,
-                            NULL, &err);
+                            NULL, &error);
 
-    if (err)
+    if (!result)
     {
-        g_warning ("%s: error in g_spawn_async", G_STRLOC);
-        g_warning ("%s: %s", G_STRLOC, err->message);
-        g_error_free (err);
+        g_warning ("error in g_spawn_async: %s", moo_error_message (error));
+        g_error_free (error);
+        error = NULL;
     }
 
     return result;
@@ -289,7 +291,7 @@ is_minimized (Display *display, XID w)
 
     if (gdk_error_trap_pop () || ret != Success)
     {
-        g_warning ("%s: oops", G_STRLOC);
+        g_warning ("oops");
         return FALSE;
     }
 
@@ -301,7 +303,7 @@ is_minimized (Display *display, XID w)
 
     if (actual_type_return != XA_ATOM)
     {
-        g_critical ("%s: actual_type_return != XA_WINDOW", G_STRLOC);
+        g_critical ("actual_type_return != XA_ATOM");
         XFree (data);
         return FALSE;
     }
@@ -359,7 +361,7 @@ _moo_get_top_window (GSList *windows)
     {
         if (!GTK_IS_WINDOW (l->data))
         {
-            g_critical ("%s: invalid parameter passed", G_STRLOC);
+            g_critical ("invalid parameter passed");
             return NULL;
         }
     }
@@ -369,7 +371,7 @@ _moo_get_top_window (GSList *windows)
 
     if (!xids->len)
     {
-        g_critical ("%s: zero length array of x ids", G_STRLOC);
+        g_critical ("zero length array of x ids");
         g_array_free (xids, TRUE);
         return NULL;
     }
@@ -378,7 +380,7 @@ _moo_get_top_window (GSList *windows)
 
     if (!display)
     {
-        g_critical ("%s: !display", G_STRLOC);
+        g_critical ("!display");
         g_array_free (xids, TRUE);
         return NULL;
     }
@@ -400,14 +402,14 @@ _moo_get_top_window (GSList *windows)
 
     if (gdk_error_trap_pop () || ret != Success)
     {
-        g_critical ("%s: error in XGetWindowProperty", G_STRLOC);
+        g_critical ("error in XGetWindowProperty");
         g_array_free (xids, TRUE);
         return NULL;
     }
 
     if (!nitems_return)
     {
-        g_critical ("%s: !nitems_return", G_STRLOC);
+        g_critical ("!nitems_return");
         XFree (data);
         g_array_free (xids, TRUE);
         return NULL;
@@ -415,7 +417,7 @@ _moo_get_top_window (GSList *windows)
 
     if (actual_type_return != XA_WINDOW)
     {
-        g_critical ("%s: actual_type_return != XA_WINDOW", G_STRLOC);
+        g_critical ("actual_type_return != XA_WINDOW");
         XFree (data);
         g_array_free (xids, TRUE);
         return NULL;
@@ -434,7 +436,7 @@ _moo_get_top_window (GSList *windows)
 
     XFree (data);
     g_array_free (xids, TRUE);
-    g_warning ("%s: all minimized?", G_STRLOC);
+    g_warning ("all minimized?");
     return GTK_WINDOW (windows->data);
 }
 
@@ -478,7 +480,7 @@ _moo_window_is_hidden (GtkWindow  *window)
         DWORD err = GetLastError ();
         char *msg = g_win32_error_message (err);
         g_return_val_if_fail (msg != NULL, FALSE);
-        g_warning ("%s: %s", G_STRLOC, msg);
+        g_warning ("%s", msg);
         g_free (msg);
         return FALSE;
     }
@@ -564,9 +566,7 @@ present_window_x11 (GtkWindow *window,
     if (!GTK_WIDGET_REALIZED (window))
         gtk_widget_realize (GTK_WIDGET (window));
 
-#if GTK_CHECK_VERSION(2,8,0)
     gdk_x11_window_move_to_current_desktop (GTK_WIDGET(window)->window);
-#endif
 
     gtk_widget_show (GTK_WIDGET (window));
 
@@ -585,7 +585,7 @@ moo_window_present (GtkWindow *window,
 
 #if defined(GDK_WINDOWING_X11)
     present_window_x11 (window, stamp);
-#elif defined(GDK_WINDOWING_QUARTZ)
+#elif defined(MOO_USE_QUARTZ)
     gtk_window_present (window);
     [[NSApplication sharedApplication] activateIgnoringOtherApps:YES];
 #else
@@ -1197,136 +1197,18 @@ _moo_get_pid_string (void)
 }
 
 
-static const char *
-moo_get_prgname (void)
+gboolean
+moo_getenv_bool (const char *var)
 {
-    const char *name;
-
-    name = g_get_prgname ();
-
-    if (!name)
-    {
-        g_critical ("%s: program name not set", G_STRLOC);
-        name = "medit";
-    }
-
-    return name;
-}
-
-
-char *
-moo_tempnam (void)
-{
-    int i;
-    char *filename = NULL;
-    static int counter;
-    G_LOCK_DEFINE_STATIC (counter);
-
-    MOO_DO_ONCE_BEGIN
-    {
-        char *dirname = NULL;
-        const char *short_name;
-
-        short_name = moo_get_prgname ();
-
-        for (i = 0; i < 1000; ++i)
-        {
-            char *basename;
-
-            basename = g_strdup_printf ("%s-%08x", short_name, g_random_int ());
-            dirname = g_build_filename (g_get_tmp_dir (), basename, NULL);
-            g_free (basename);
-
-            if (_moo_mkdir (dirname) == 0)
-                break;
-
-            g_free (dirname);
-            dirname = NULL;
-        }
-
-        moo_temp_dir = dirname;
-        moo_install_atexit ();
-    }
-    MOO_DO_ONCE_END
-
-    g_return_val_if_fail (moo_temp_dir != NULL, NULL);
-
-    G_LOCK (counter);
-
-    for (i = counter + 1; i < counter + 1000; ++i)
-    {
-        char *basename;
-
-        basename = g_strdup_printf ("tmpfile-%03d", i);
-        filename = g_build_filename (moo_temp_dir, basename, NULL);
-        g_free (basename);
-
-        if (!g_file_test (filename, G_FILE_TEST_EXISTS))
-            break;
-
-        g_free (filename);
-        filename = NULL;
-    }
-
-    counter = i;
-
-    G_UNLOCK (counter);
-
-    if (!filename)
-        g_warning ("%s: could not generate temp file name", G_STRLOC);
-
-    return filename;
-}
-
-static void
-moo_remove_tempdir (void)
-{
-    if (moo_temp_dir)
-    {
-        GError *error = NULL;
-        _moo_remove_dir (moo_temp_dir, TRUE, &error);
-
-        if (error)
-        {
-            _moo_message ("%s: %s", G_STRLOC, error->message);
-            g_error_free (error);
-        }
-
-        g_free (moo_temp_dir);
-        moo_temp_dir = NULL;
-    }
-}
-
-static void
-moo_atexit_handler (void)
-{
-    moo_remove_tempdir ();
-}
-
-static void
-moo_install_atexit (void)
-{
-    static gboolean installed;
-
-    if (!installed)
-    {
-        atexit (moo_atexit_handler);
-        installed = TRUE;
-    }
-}
-
-
-void
-moo_cleanup (void)
-{
-    moo_remove_tempdir ();
+    const char *value = g_getenv (var);
+    return value && *value && strcmp (value, "0") != 0;
 }
 
 
 char *
 moo_get_user_cache_dir (void)
 {
-    return g_build_filename (g_get_user_cache_dir (), moo_get_prgname (), NULL);
+    return g_build_filename (g_get_user_cache_dir (), MOO_PACKAGE_NAME, NULL);
 }
 
 char *
@@ -1343,7 +1225,7 @@ moo_get_user_data_dir (void)
 #endif
 
         moo_user_data_dir = g_build_filename (basedir,
-                                              moo_get_prgname (),
+                                              MOO_PACKAGE_NAME,
                                               NULL);
     }
 
@@ -1358,7 +1240,7 @@ moo_set_user_data_dir (const char *path)
     G_LOCK (moo_user_data_dir);
 
     if (moo_user_data_dir)
-        g_critical ("%s: user data dir already set", G_STRLOC);
+        g_critical ("user data dir already set");
 
     g_free (moo_user_data_dir);
     moo_user_data_dir = g_strdup (path);
@@ -1381,6 +1263,20 @@ _moo_set_app_instance_name (const char *name)
     moo_app_instance_name = g_strdup (name);
 }
 
+void
+moo_set_display_app_name (const char *name)
+{
+    g_return_if_fail (name && *name);
+    g_free (moo_display_app_name);
+    moo_display_app_name = g_strdup (name);
+}
+
+const char *
+moo_get_display_app_name (void)
+{
+    return moo_display_app_name ? moo_display_app_name : g_get_prgname ();
+}
+
 
 gboolean
 moo_make_user_data_dir (const char *path)
@@ -1408,6 +1304,12 @@ moo_make_user_data_dir (const char *path)
 }
 
 
+typedef enum {
+    MOO_DATA_SHARE,
+    MOO_DATA_LIB,
+    MOO_DATA_BOTH
+} MooDataDirType;
+
 static gboolean
 cmp_dirs (const char *dir1,
           const char *dir2)
@@ -1418,7 +1320,6 @@ cmp_dirs (const char *dir1,
     return !strcmp (dir1, dir2);
 }
 
-
 static void
 add_dir_list_from_env (GPtrArray  *list,
                        const char *var)
@@ -1440,126 +1341,149 @@ add_dir_list_from_env (GPtrArray  *list,
     g_free (dirs);
 }
 
-
-static char **
-moo_get_data_dirs_real (MooDataDirType   type,
-                        char           **add,
-                        guint           *n_dirs)
+static void
+enumerate_data_dirs (MooDataDirType  type,
+                     GPtrArray      *dirs)
 {
-    static char **moo_data_dirs[2];
-    static guint n_data_dirs[2];
-    G_LOCK_DEFINE_STATIC(moo_data_dirs);
+    const char *env[2];
 
-    g_return_val_if_fail (type < 2, NULL);
-
-    G_LOCK (moo_data_dirs);
-
-    if (add && moo_data_dirs[type])
-        g_critical ("moo_add_data_dirs() called too late, ignored");
-
-    if (!moo_data_dirs[type])
+    if (type == MOO_DATA_BOTH)
     {
-        const char *env[2];
-        GPtrArray *dirs;
-        GPtrArray *all_dirs;
-        char **ptr;
-        guint i;
+        enumerate_data_dirs (MOO_DATA_LIB, dirs);
+        enumerate_data_dirs (MOO_DATA_SHARE, dirs);
+        return;
+    }
 
-        all_dirs = g_ptr_array_new ();
-        dirs = g_ptr_array_new ();
+    g_ptr_array_add (dirs, moo_get_user_data_dir ());
 
-        env[0] = g_getenv ("MOO_APP_DIRS");
-        env[1] = type == MOO_DATA_SHARE ? g_getenv ("MOO_DATA_DIRS") : g_getenv ("MOO_LIB_DIRS");
+    env[0] = g_getenv ("MOO_APP_DIRS");
+    env[1] = type == MOO_DATA_SHARE ? g_getenv ("MOO_DATA_DIRS") : g_getenv ("MOO_LIB_DIRS");
 
-        g_ptr_array_add (all_dirs, moo_get_user_data_dir ());
-
-        while (add && *add)
-            g_ptr_array_add (all_dirs, g_strdup (*add++));
-
-        /* environment variables override everything */
-        if (env[0] || env[1])
+    /* environment variables override everything */
+    if (env[0] || env[1])
+    {
+        if (env[1])
+            add_dir_list_from_env (dirs, env[1]);
+        else
+            add_dir_list_from_env (dirs, env[0]);
+    }
+    else
+    {
+#ifdef __WIN32__
+        _moo_win32_add_data_dirs (dirs, type == MOO_DATA_SHARE ? "share" : "lib");
+#else
+        if (type == MOO_DATA_SHARE)
         {
-            if (env[1])
-                add_dir_list_from_env (all_dirs, env[1]);
-            else
-                add_dir_list_from_env (all_dirs, env[0]);
+            const char* const *p;
+
+            for (p = g_get_system_data_dirs (); p && *p; ++p)
+                g_ptr_array_add (dirs, g_build_filename (*p, MOO_PACKAGE_NAME, NULL));
+
+            g_ptr_array_add (dirs, g_strdup (MOO_DATA_DIR));
         }
         else
         {
-#ifdef __WIN32__
-            _moo_win32_add_data_dirs (all_dirs, type == MOO_DATA_SHARE ? "share" : "lib");
-#else
-            if (type == MOO_DATA_SHARE)
-            {
-                const char* const *p;
-
-                for (p = g_get_system_data_dirs (); p && *p; ++p)
-                    g_ptr_array_add (all_dirs, g_build_filename (*p, MOO_PACKAGE_NAME, NULL));
-
-                g_ptr_array_add (all_dirs, g_strdup (MOO_DATA_DIR));
-            }
-            else
-            {
-                g_ptr_array_add (all_dirs, g_strdup (MOO_LIB_DIR));
-            }
+            g_ptr_array_add (dirs, g_strdup (MOO_LIB_DIR));
+        }
 #endif
-        }
+    }
+}
 
-        g_ptr_array_add (all_dirs, NULL);
+static char **
+do_get_data_dirs (MooDataDirType  type,
+                  guint          *n_dirs)
+{
+    GPtrArray *dirs;
+    GPtrArray *all_dirs;
+    char **ptr;
+    guint i;
 
-        for (ptr = (char**) all_dirs->pdata; *ptr; ++ptr)
+    all_dirs = g_ptr_array_new ();
+    enumerate_data_dirs (type, all_dirs);
+    g_ptr_array_add (all_dirs, NULL);
+
+    dirs = g_ptr_array_new ();
+
+    for (ptr = (char**) all_dirs->pdata; *ptr; ++ptr)
+    {
+        gboolean found = FALSE;
+        char *path;
+
+        path = *ptr;
+
+        if (!path || !path[0])
         {
-            gboolean found = FALSE;
-            char *path;
-
-            path = *ptr;
-
-            if (!path || !path[0])
-            {
-                g_free (path);
-                continue;
-            }
-
-            for (i = 0; i < dirs->len; ++i)
-            {
-                if (cmp_dirs (path, dirs->pdata[i]))
-                {
-                    found = TRUE;
-                    break;
-                }
-            }
-
-            if (!found)
-                g_ptr_array_add (dirs, path);
-            else
-                g_free (path);
+            g_free (path);
+            continue;
         }
 
-        g_ptr_array_add (dirs, NULL);
-        n_data_dirs[type] = dirs->len - 1;
-        moo_data_dirs[type] = (char**) g_ptr_array_free (dirs, FALSE);
-        g_ptr_array_free (all_dirs, TRUE);
+        for (i = 0; i < dirs->len; ++i)
+        {
+            if (cmp_dirs (path, dirs->pdata[i]))
+            {
+                found = TRUE;
+                break;
+            }
+        }
+
+        if (!found)
+            g_ptr_array_add (dirs, path);
+        else
+            g_free (path);
+    }
+
+    g_ptr_array_add (dirs, NULL);
+    *n_dirs = dirs->len - 1;
+    g_ptr_array_free (all_dirs, TRUE);
+    return (char**) g_ptr_array_free (dirs, FALSE);
+}
+
+static char **
+moo_get_data_dirs_real (MooDataDirType   type_requested,
+                        gboolean         include_user,
+                        guint           *n_dirs)
+{
+    static char **moo_data_dirs[3];
+    static guint n_data_dirs[3];
+    G_LOCK_DEFINE_STATIC(moo_data_dirs);
+
+    g_return_val_if_fail (type_requested < 3, NULL);
+
+    G_LOCK (moo_data_dirs);
+
+    if (!moo_data_dirs[0])
+    {
+        int type;
+        for (type = 0; type < 3; ++type)
+            moo_data_dirs[type] = do_get_data_dirs (type, &n_data_dirs[type]);
     }
 
     G_UNLOCK (moo_data_dirs);
 
-    if (n_dirs)
-        *n_dirs = n_data_dirs[type];
-
-    return g_strdupv (moo_data_dirs[type]);
+    if (include_user || !n_data_dirs[type_requested])
+    {
+        if (n_dirs)
+            *n_dirs = n_data_dirs[type_requested];
+        return g_strdupv (moo_data_dirs[type_requested]);
+    }
+    else
+    {
+        if (n_dirs)
+            *n_dirs = n_data_dirs[type_requested] - 1;
+        return g_strdupv (moo_data_dirs[type_requested] + 1);
+    }
 }
 
 char **
-moo_get_data_dirs (MooDataDirType type,
-                   guint         *n_dirs)
+moo_get_data_dirs (void)
 {
-    return moo_get_data_dirs_real (type, NULL, n_dirs);
+    return moo_get_data_dirs_real (MOO_DATA_SHARE, TRUE, NULL);
 }
 
-void
-moo_add_data_dirs (char **dirs)
+char **
+moo_get_lib_dirs (void)
 {
-    g_strfreev (moo_get_data_dirs_real (MOO_DATA_SHARE, dirs, NULL));
+    return moo_get_data_dirs_real (MOO_DATA_LIB, TRUE, NULL);
 }
 
 
@@ -1575,22 +1499,19 @@ moo_get_locale_dir (void)
 }
 
 
-char **
-moo_get_data_subdirs (const char    *subdir,
-                      MooDataDirType type,
-                      guint         *n_dirs_p)
+static char **
+moo_get_stuff_subdirs (const char    *subdir,
+                       MooDataDirType type,
+                       gboolean       include_user)
 {
     char **data_dirs, **dirs;
     guint n_dirs, i;
 
     g_return_val_if_fail (subdir != NULL, NULL);
 
-    data_dirs = moo_get_data_dirs (type, &n_dirs);
+    data_dirs = moo_get_data_dirs_real (type, include_user, &n_dirs);
     g_return_val_if_fail (data_dirs != NULL, NULL);
 
-    if (n_dirs_p)
-        *n_dirs_p = n_dirs;
-
     dirs = g_new0 (char*, n_dirs + 1);
 
     for (i = 0; i < n_dirs; ++i)
@@ -1600,6 +1521,30 @@ moo_get_data_subdirs (const char    *subdir,
     return dirs;
 }
 
+char **
+moo_get_data_subdirs (const char *subdir)
+{
+    return moo_get_stuff_subdirs (subdir, MOO_DATA_SHARE, TRUE);
+}
+
+char **
+moo_get_sys_data_subdirs (const char *subdir)
+{
+    return moo_get_stuff_subdirs (subdir, MOO_DATA_SHARE, FALSE);
+}
+
+char **
+moo_get_lib_subdirs (const char *subdir)
+{
+    return moo_get_stuff_subdirs (subdir, MOO_DATA_LIB, TRUE);
+}
+
+char **
+moo_get_data_and_lib_subdirs (const char *subdir)
+{
+    return moo_get_stuff_subdirs (subdir, MOO_DATA_BOTH, TRUE);
+}
+
 
 static char *
 get_user_data_file (const char *basename,
@@ -1656,8 +1601,7 @@ moo_get_user_cache_file (const char *basename)
 
 
 static gboolean
-save_config_file (const char     *dir,
-                  const char     *filename,
+save_config_file (const char     *filename,
                   const char     *content,
                   gssize          len,
                   GError        **error)
@@ -1665,7 +1609,6 @@ save_config_file (const char     *dir,
     MooFileWriter *writer;
     gboolean retval;
 
-    g_return_val_if_fail (dir != NULL, FALSE);
     g_return_val_if_fail (filename != NULL, FALSE);
     g_return_val_if_fail (content != NULL, FALSE);
 
@@ -1685,26 +1628,19 @@ save_user_data_file (const char     *basename,
                      gssize          len,
                      GError        **error)
 {
-    char *dir, *file;
+    char *file;
     gboolean result;
 
     g_return_val_if_fail (basename != NULL, FALSE);
     g_return_val_if_fail (content != NULL, FALSE);
 
     if (cache)
-    {
-        dir = moo_get_user_cache_dir ();
         file = moo_get_user_cache_file (basename);
-    }
     else
-    {
-        dir = moo_get_user_data_dir ();
         file = moo_get_user_data_file (basename);
-    }
 
-    result = save_config_file (dir, file, content, len, error);
+    result = save_config_file (file, content, len, error);
 
-    g_free (dir);
     g_free (file);
     return result;
 }
@@ -1733,17 +1669,13 @@ moo_save_config_file (const char   *filename,
                       gssize        len,
                       GError      **error)
 {
-    char *dir;
     gboolean result;
 
     g_return_val_if_fail (filename != NULL, FALSE);
     g_return_val_if_fail (content != NULL, FALSE);
 
-    dir = g_path_get_dirname (filename);
+    result = save_config_file (filename, content, len, error);
 
-    result = save_config_file (dir, filename, content, len, error);
-
-    g_free (dir);
     return result;
 }
 
@@ -1752,26 +1684,12 @@ void
 _moo_widget_set_tooltip (GtkWidget  *widget,
                          const char *tip)
 {
-#if !GTK_CHECK_VERSION(2,11,6)
-    static GtkTooltips *tooltips;
-
-    g_return_if_fail (GTK_IS_WIDGET (widget));
-
-    if (!tooltips)
-        tooltips = gtk_tooltips_new ();
-
-    if (GTK_IS_TOOL_ITEM (widget))
-        gtk_tool_item_set_tooltip (GTK_TOOL_ITEM (widget), tooltips, tip, NULL);
-    else
-        gtk_tooltips_set_tip (tooltips, widget, tip, tip);
-#else
     g_return_if_fail (GTK_IS_WIDGET (widget));
 
     if (GTK_IS_TOOL_ITEM (widget))
         gtk_tool_item_set_tooltip_text (GTK_TOOL_ITEM (widget), tip);
     else
         gtk_widget_set_tooltip_text (widget, tip);
-#endif
 }
 
 
@@ -1785,10 +1703,9 @@ void
 _moo_abort_debug_ignore (MooCodeLoc loc, const char *message)
 {
     if (moo_code_loc_valid (loc))
-        g_printerr ("In file %s, line %d, function %s:\n%s\n", loc.file, loc.line, loc.func, message);
+	g_error ("file '%s', function '%s', line %d: %s\n", loc.file, loc.func, loc.line, message);
     else
-        g_printerr ("%s\n", message);
-    g_on_error_query (moo_get_prgname());
+        g_error ("%s\n", message);
 }
 
 #else /* !__WIN32__ */
@@ -1828,7 +1745,7 @@ _moo_abort_debug_ignore (MooCodeLoc loc, const char *message)
 
         if (use_location)
             ret = _moo_win32_message_box (NULL, MB_ABORTRETRYIGNORE, NULL,
-                                          "In file %s, line %d, function %s:\n%s",
+                                          Q_("console message|in file %s, line %d, function %s:\n%s"),
                                           loc.file, loc.line, loc.func, message);
         else
             ret = _moo_win32_message_box (NULL, MB_ABORTRETRYIGNORE, NULL,
@@ -1866,14 +1783,16 @@ _moo_abort_debug_ignore (MooCodeLoc loc, const char *message)
 
 #endif /* !__WIN32__ */
 
-#ifndef MOO_DEV_MODE
+#if !defined(MOO_DEV_MODE) && !defined(DEBUG)
 NORETURN
 #endif
 void
 _moo_assert_message (MooCodeLoc loc, const char *message)
 {
-#if defined(MOO_DEV_MODE) && !defined(__WIN32__)
+#if defined(MOO_DEV_MODE) && defined(__WIN32__)
     _moo_abort_debug_ignore (loc, message);
+#elif defined(MOO_DEV_MODE) || defined(DEBUG)
+    g_critical ("file '%s', function '%s', line %d: %s\n", loc.file, loc.func, loc.line, message);
 #else
     g_error ("file '%s', function '%s', line %d: %s\n", loc.file, loc.func, loc.line, message);
 #endif
@@ -1908,11 +1827,13 @@ moo_log_debug_enabled (void)
 
 void _moo_logv (MooCodeLoc loc, GLogLevelFlags flags, const char *format, va_list args)
 {
-    char *message = g_strdup_vprintf (format, args);
+    char *message;
 
     if (flags >= G_LOG_LEVEL_DEBUG && !moo_log_debug_enabled ())
         return;
 
+    message = g_strdup_vprintf (format, args);
+
 #if defined(MOO_DEV_MODE) && !defined(__WIN32__)
     if (flags < G_LOG_LEVEL_MESSAGE)
     {
@@ -1923,7 +1844,7 @@ void _moo_logv (MooCodeLoc loc, GLogLevelFlags flags, const char *format, va_lis
 
     if (moo_code_loc_valid (loc))
         g_log (G_LOG_DOMAIN, flags,
-               "In file %s, line %d, function %s: %s",
+               Q_("console message|in file %s, line %d, function %s: %s"),
                loc.file, loc.line, loc.func, message);
     else
         g_log (G_LOG_DOMAIN, flags, "%s", message);
@@ -2211,14 +2132,6 @@ _moo_strv_reverse (char **str_array)
 }
 
 
-gboolean
-_moo_str_equal (const char *s1,
-                const char *s2)
-{
-    return strcmp (s1 ? s1 : "", s2 ? s2 : "") == 0;
-}
-
-
 #if defined(__WIN32__) && !defined(MOO_DEBUG)
 static guint saved_win32_error_mode;
 #endif
@@ -2240,17 +2153,6 @@ moo_enable_win32_error_message (void)
 }
 
 
-#if 0
-#undef moo_str_equal
-gboolean
-moo_str_equal (const char *s1,
-               const char *s2)
-{
-    return !strcmp (s1 ? s1 : "", s2 ? s2 : "");
-}
-#endif
-
-
 typedef struct {
     GSourceFunc func;
     gpointer data;
@@ -2266,80 +2168,6 @@ source_data_free (gpointer data)
     g_free (sd);
 }
 
-static gboolean
-thread_source_func (gpointer data)
-{
-    SourceData *sd = data;
-    gboolean ret = FALSE;
-
-    gdk_threads_enter ();
-
-#if GLIB_CHECK_VERSION(2,12,0)
-    if (!g_source_is_destroyed (g_main_current_source ()))
-#endif
-        ret = sd->func (sd->data);
-
-    gdk_threads_leave ();
-    return ret;
-}
-
-guint
-moo_idle_add_full (gint           priority,
-                   GSourceFunc    function,
-                   gpointer       data,
-                   GDestroyNotify notify)
-{
-    SourceData *sd;
-
-    sd = g_new (SourceData, 1);
-    sd->func = function;
-    sd->data = data;
-    sd->notify = notify;
-
-    return g_idle_add_full (priority, thread_source_func,
-                            sd, source_data_free);
-}
-
-
-guint
-moo_idle_add (GSourceFunc function,
-              gpointer    data)
-{
-    return moo_idle_add_full (G_PRIORITY_DEFAULT_IDLE,
-                              function, data, NULL);
-}
-
-
-guint
-_moo_timeout_add_full (gint           priority,
-                       guint          interval,
-                       GSourceFunc    function,
-                       gpointer       data,
-                       GDestroyNotify notify)
-{
-    SourceData *sd;
-
-    sd = g_new (SourceData, 1);
-    sd->func = function;
-    sd->data = data;
-    sd->notify = notify;
-
-    return g_timeout_add_full (priority, interval,
-                               thread_source_func,
-                               sd, source_data_free);
-}
-
-
-guint
-_moo_timeout_add (guint       interval,
-                  GSourceFunc function,
-                  gpointer    data)
-{
-    return _moo_timeout_add_full (G_PRIORITY_DEFAULT, interval,
-                                  function, data, NULL);
-}
-
-
 static gboolean
 thread_io_func (GIOChannel  *source,
                 GIOCondition condition,
@@ -2350,9 +2178,7 @@ thread_io_func (GIOChannel  *source,
 
     gdk_threads_enter ();
 
-#if GLIB_CHECK_VERSION(2,12,0)
     if (!g_source_is_destroyed (g_main_current_source ()))
-#endif
         ret = ((GIOFunc) sd->func) (source, condition, sd->data);
 
     gdk_threads_leave ();
@@ -2390,35 +2216,6 @@ _moo_io_add_watch_full (GIOChannel    *channel,
 }
 
 
-const char *
-_moo_intern_string (const char *string)
-{
-    if (!string)
-        return NULL;
-
-#if GLIB_CHECK_VERSION(2,10,0)
-    return g_intern_string (string);
-#else
-    {
-        static GHashTable *hash;
-        gpointer original;
-        gpointer dummy;
-
-        if (G_UNLIKELY (!hash))
-            hash = g_hash_table_new (g_str_hash, g_str_equal);
-
-        if (!g_hash_table_lookup_extended (hash, string, &original, &dummy))
-        {
-            char *copy = g_strdup (string);
-            g_hash_table_insert (hash, copy, NULL);
-            original = copy;
-        }
-
-        return original;
-    }
-#endif
-}
-
 GdkAtom
 moo_atom_uri_list (void)
 {
@@ -2426,6 +2223,42 @@ moo_atom_uri_list (void)
 }
 
 
+const char *
+moo_error_message (GError *error)
+{
+    /* "Unknown error" may be displayed if there is a bug in the program
+       and proper error message has not been set. It doesn't provide any
+       useful information obviously */
+    g_return_val_if_fail (error != NULL, _("Unknown error"));
+    return error->message;
+}
+
+
+gboolean
+moo_signal_accumulator_continue_cancel (G_GNUC_UNUSED GSignalInvocationHint *ihint,
+                                        GValue                *return_accu,
+                                        const GValue          *handler_return,
+                                        gpointer               val_continue)
+{
+    int ret = g_value_get_enum (handler_return);
+    int ival_continue = GPOINTER_TO_INT (val_continue);
+    int ival_cancel = ival_continue + 1;
+
+    if (ret == ival_cancel)
+    {
+        g_value_set_enum (return_accu, ival_cancel);
+        return FALSE;
+    }
+    else
+    {
+        g_value_set_enum (return_accu, ival_continue);
+        if (ret != ival_continue)
+            g_critical ("invalid return value %d", ret);
+        return TRUE;
+    }
+}
+
+
 static char *debug_domains;
 void _moo_set_debug (const char *domains);
 gboolean moo_debug_enabled (const char *domain, gboolean def_enabled);
@@ -2470,10 +2303,6 @@ moo_debug_enabled (const char *domain,
 }
 
 
-#ifdef MOO_ENABLE_UNIT_TESTS
-
-#include <mooutils/mooutils-tests.h>
-
 static void
 test_strv_one (const char *string,
                gssize      len,
@@ -2548,15 +2377,27 @@ test_moo_splitlines (void)
 }
 
 
+static void
+test_types (void)
+{
+    TEST_ASSERT (g_type_is_a (MOO_TYPE_SAVE_CHANGES_RESPONSE, G_TYPE_ENUM));
+    TEST_ASSERT (g_type_is_a (MOO_TYPE_FILE_DIALOG_TYPE, G_TYPE_ENUM));
+    TEST_ASSERT (g_type_is_a (MOO_TYPE_UI_NODE_TYPE, G_TYPE_ENUM));
+    TEST_ASSERT (g_type_is_a (MOO_TYPE_UI_NODE_FLAGS, G_TYPE_FLAGS));
+    TEST_ASSERT (g_type_is_a (MOO_TYPE_UI_WIDGET_TYPE, G_TYPE_ENUM));
+    TEST_ASSERT (g_type_is_a (MOO_TYPE_CLOSE_RESPONSE, G_TYPE_ENUM));
+}
+
+
 void
 moo_test_mooutils_misc (void)
 {
     MooTestSuite *suite;
 
-    suite = moo_test_suite_new ("mooutils/mooutils-misc.c", NULL, NULL, NULL);
+    suite = moo_test_suite_new ("mooutils-misc", "mooutils/mooutils-misc.c", NULL, NULL, NULL);
 
-    moo_test_suite_add_test (suite, "test of moo_splitlines()",
+    moo_test_suite_add_test (suite, "moo_splitlines", "test of moo_splitlines()",
                              (MooTestFunc) test_moo_splitlines, NULL);
+    moo_test_suite_add_test (suite, "types", "sanity checks for Glib types",
+                             (MooTestFunc) test_types, NULL);
 }
-
-#endif
