@@ -1,15 +1,22 @@
 /*
  *   moowindow.c
  *
- *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
+ *   Copyright (C) 2004-2010 by Yevgen Muntyan <emuntyan@users.sourceforge.net>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+/**
+ * class:MooWindow: (parent GtkWindow)
+ **/
+
 #include "mooutils/moowindow.h"
 #include "mooutils/mooaction-private.h"
 #include "mooutils/mooactionbase-private.h"
@@ -24,6 +31,9 @@
 #include "mooutils/mooutils-misc.h"
 #include "mooutils/mooutils-mem.h"
 #include "mooutils/mooeditops.h"
+#include "mooutils/mootype-macros.h"
+#include "mooutils/mooutils-enums.h"
+#include "mooutils/moocompat.h"
 #include <gtk/gtk.h>
 #include <gobject/gvaluecollector.h>
 
@@ -39,6 +49,7 @@
 
 #define TOOLBAR_STYLE_ACTION_ID "ToolbarStyle"
 
+static char *default_geometry = NULL;
 
 static GSList *window_instances = NULL;
 
@@ -55,19 +66,27 @@ struct _MooWindowPrivate {
 
     gboolean statusbar_visible;
 
-    MooUIXML *ui_xml;
+    MooUiXml *ui_xml;
     MooActionCollection *actions;
     char *name;
     char *id;
 
+    GSList *global_accels;
+    guint update_accels_idle;
+
     GtkWidget *eo_widget;
     GtkWidget *default_eo_widget;
     GtkWidget *uo_widget;
     GtkWidget *default_uo_widget;
 
-    guint global_accels : 1;
+    guint global_accels_mode : 1;
 };
 
+typedef struct {
+    guint keyval;
+    GdkModifierType modifiers;
+    GtkAction *action;
+} AccelEntry;
 
 static const char *setting (MooWindow *window, const char *s)
 {
@@ -136,6 +155,8 @@ static void         moo_window_set_statusbar_visible    (MooWindow      *window,
 static GtkAction   *create_toolbar_style_action         (MooWindow      *window,
                                                          gpointer        dummy);
 
+static MooCloseResponse moo_window_close_handler        (MooWindow      *window);
+
 static void         moo_window_set_focus                (GtkWindow      *window,
                                                          GtkWidget      *widget);
 static void         moo_window_disconnect_eo_widget     (MooWindow      *window);
@@ -148,6 +169,9 @@ static void         moo_window_action_select_all        (MooWindow      *window)
 static void         moo_window_action_undo              (MooWindow      *window);
 static void         moo_window_action_redo              (MooWindow      *window);
 
+static void         accel_entry_free                    (AccelEntry     *entry);
+static void         accels_changed                      (MooWindow      *window);
+
 
 enum {
     PROP_0,
@@ -156,6 +180,8 @@ enum {
     PROP_TOOLBAR_UI_NAME,
     PROP_ID,
     PROP_UI_XML,
+    PROP_MENUBAR,
+    PROP_TOOLBAR,
     PROP_ACTIONS,
     PROP_TOOLBAR_VISIBLE,
     PROP_MENUBAR_VISIBLE,
@@ -173,7 +199,6 @@ enum {
 
 enum {
     CLOSE,
-    APPLY_PREFS,
     LAST_SIGNAL
 };
 
@@ -208,6 +233,8 @@ moo_window_class_init (MooWindowClass *klass)
     widget_class->key_press_event = moo_window_key_press_event;
     window_class->set_focus = moo_window_set_focus;
 
+    klass->close = moo_window_close_handler;
+
     moo_window_class_set_id (klass, "MooWindow", "Window");
 
     _moo_edit_ops_iface_install ();
@@ -231,6 +258,7 @@ moo_window_class_init (MooWindowClass *klass)
                                  "display-name", _("Show Menubar"),
                                  "label", _("Show Menubar"),
                                  "condition::active", "menubar-visible",
+                                 "connect-accel", TRUE,
                                  NULL);
 
     moo_window_class_new_action (klass, "ShowStatusbar", NULL,
@@ -249,7 +277,7 @@ moo_window_class_init (MooWindowClass *klass)
                                  "label", GTK_STOCK_CUT,
                                  "tooltip", GTK_STOCK_CUT,
                                  "stock-id", GTK_STOCK_CUT,
-                                 "accel", MOO_ACCEL_CUT,
+                                 "default-accel", MOO_ACCEL_CUT,
                                  "closure-callback", moo_window_action_cut,
                                  "condition::sensitive", "can-cut",
                                  NULL);
@@ -259,7 +287,7 @@ moo_window_class_init (MooWindowClass *klass)
                                  "label", GTK_STOCK_COPY,
                                  "tooltip", GTK_STOCK_COPY,
                                  "stock-id", GTK_STOCK_COPY,
-                                 "accel", MOO_ACCEL_COPY,
+                                 "default-accel", MOO_ACCEL_COPY,
                                  "closure-callback", moo_window_action_copy,
                                  "condition::sensitive", "can-copy",
                                  NULL);
@@ -269,7 +297,7 @@ moo_window_class_init (MooWindowClass *klass)
                                  "label", GTK_STOCK_PASTE,
                                  "tooltip", GTK_STOCK_PASTE,
                                  "stock-id", GTK_STOCK_PASTE,
-                                 "accel", MOO_ACCEL_PASTE,
+                                 "default-accel", MOO_ACCEL_PASTE,
                                  "closure-callback", moo_window_action_paste,
                                  "condition::sensitive", "can-paste",
                                  NULL);
@@ -288,7 +316,7 @@ moo_window_class_init (MooWindowClass *klass)
                                  "label", GTK_STOCK_SELECT_ALL,
                                  "tooltip", GTK_STOCK_SELECT_ALL,
                                  "stock-id", GTK_STOCK_SELECT_ALL,
-                                 "accel", MOO_ACCEL_SELECT_ALL,
+                                 "default-accel", MOO_ACCEL_SELECT_ALL,
                                  "closure-callback", moo_window_action_select_all,
                                  "condition::sensitive", "can-select-all",
                                  NULL);
@@ -298,7 +326,7 @@ moo_window_class_init (MooWindowClass *klass)
                                  "label", GTK_STOCK_UNDO,
                                  "tooltip", GTK_STOCK_UNDO,
                                  "stock-id", GTK_STOCK_UNDO,
-                                 "accel", MOO_ACCEL_UNDO,
+                                 "default-accel", MOO_ACCEL_UNDO,
                                  "closure-callback", moo_window_action_undo,
                                  "condition::sensitive", "can-undo",
                                  NULL);
@@ -308,7 +336,7 @@ moo_window_class_init (MooWindowClass *klass)
                                  "label", GTK_STOCK_REDO,
                                  "tooltip", GTK_STOCK_REDO,
                                  "stock-id", GTK_STOCK_REDO,
-                                 "accel", MOO_ACCEL_REDO,
+                                 "default-accel", MOO_ACCEL_REDO,
                                  "closure-callback", moo_window_action_redo,
                                  "condition::sensitive", "can-redo",
                                  NULL);
@@ -319,11 +347,11 @@ moo_window_class_init (MooWindowClass *klass)
 
     g_object_class_install_property (gobject_class, PROP_MENUBAR_UI_NAME,
         g_param_spec_string ("menubar-ui-name", "menubar-ui-name", "menubar-ui-name",
-                             NULL, G_PARAM_READWRITE));
+                             NULL, (GParamFlags) G_PARAM_READWRITE));
 
     g_object_class_install_property (gobject_class, PROP_TOOLBAR_UI_NAME,
         g_param_spec_string ("toolbar-ui-name", "toolbar-ui-name", "toolbar-ui-name",
-                             NULL, G_PARAM_READWRITE));
+                             NULL, (GParamFlags) G_PARAM_READWRITE));
 
     g_object_class_install_property (gobject_class, PROP_ID,
         g_param_spec_string ("id", "id", "id", NULL, G_PARAM_READABLE));
@@ -334,19 +362,27 @@ moo_window_class_init (MooWindowClass *klass)
 
     g_object_class_install_property (gobject_class, PROP_UI_XML,
         g_param_spec_object ("ui-xml", "ui-xml", "ui-xml",
-                             MOO_TYPE_UI_XML, G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+                             MOO_TYPE_UI_XML, (GParamFlags) (G_PARAM_READWRITE | G_PARAM_CONSTRUCT)));
+
+    g_object_class_install_property (gobject_class, PROP_TOOLBAR,
+        g_param_spec_object ("toolbar", "toolbar", "toolbar",
+                             GTK_TYPE_TOOLBAR, G_PARAM_READABLE));
+
+    g_object_class_install_property (gobject_class, PROP_MENUBAR,
+        g_param_spec_object ("menubar", "menubar", "menubar",
+                             GTK_TYPE_MENU, G_PARAM_READABLE));
 
     g_object_class_install_property (gobject_class, PROP_TOOLBAR_VISIBLE,
         g_param_spec_boolean ("toolbar-visible", "toolbar-visible", "toolbar-visible",
-                              TRUE, G_PARAM_READWRITE));
+                              TRUE, (GParamFlags) G_PARAM_READWRITE));
 
     g_object_class_install_property (gobject_class, PROP_MENUBAR_VISIBLE,
         g_param_spec_boolean ("menubar-visible", "menubar-visible", "menubar-visible",
-                              TRUE, G_PARAM_READWRITE));
+                              TRUE, (GParamFlags) G_PARAM_READWRITE));
 
     g_object_class_install_property (gobject_class, PROP_STATUSBAR_VISIBLE,
         g_param_spec_boolean ("statusbar-visible", "statusbar-visible", "statusbar-visible",
-                              TRUE, G_PARAM_READWRITE));
+                              TRUE, (GParamFlags) G_PARAM_READWRITE));
 
     INSTALL_PROP (PROP_MEO_CAN_COPY, "can-copy");
     INSTALL_PROP (PROP_MEO_CAN_CUT, "can-cut");
@@ -362,18 +398,10 @@ moo_window_class_init (MooWindowClass *klass)
                       G_OBJECT_CLASS_TYPE (klass),
                       G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (MooWindowClass, close),
-                      g_signal_accumulator_true_handled, NULL,
-                      _moo_marshal_BOOL__VOID,
-                      G_TYPE_BOOLEAN, 0);
-
-    signals[APPLY_PREFS] =
-            g_signal_new ("apply-prefs",
-                      G_OBJECT_CLASS_TYPE (klass),
-                      G_SIGNAL_RUN_LAST,
-                      G_STRUCT_OFFSET (MooWindowClass, apply_prefs),
-                      NULL, NULL,
-                      _moo_marshal_VOID__VOID,
-                      G_TYPE_NONE, 0);
+                      moo_signal_accumulator_continue_cancel,
+                      GINT_TO_POINTER (MOO_CLOSE_RESPONSE_CONTINUE),
+                      _moo_marshal_ENUM__VOID,
+                      MOO_TYPE_CLOSE_RESPONSE, 0);
 }
 
 
@@ -427,7 +455,12 @@ moo_window_constructor (GType                  type,
     g_signal_connect (window, "notify::ui-xml",
                       G_CALLBACK (moo_window_update_ui), NULL);
 
-    if (moo_prefs_get_bool (setting (window, PREFS_REMEMBER_SIZE)))
+    if (default_geometry && *default_geometry)
+    {
+        if (!gtk_window_parse_geometry (GTK_WINDOW (window), default_geometry))
+            g_printerr (_("Could not parse geometry string '%s'\n"), default_geometry);
+    }
+    else if (moo_prefs_get_bool (setting (window, PREFS_REMEMBER_SIZE)))
     {
         int width = moo_prefs_get_int (setting (window, PREFS_WIDTH));
         int height = moo_prefs_get_int (setting (window, PREFS_HEIGHT));
@@ -458,11 +491,22 @@ moo_window_constructor (GType                  type,
 
     moo_window_update_ui (window);
 
+    accels_changed (window);
+
     g_type_class_unref (klass);
     return object;
 }
 
 
+void
+moo_window_set_default_geometry (const char *geometry)
+{
+    char *tmp = default_geometry;
+    default_geometry = g_strdup (geometry);
+    g_free (tmp);
+}
+
+
 static void
 parse_shadow_style (void)
 {
@@ -495,7 +539,7 @@ moo_window_init (MooWindow *window)
     window->status_area = gtk_hbox_new (FALSE, 0);
     window->statusbar = g_object_new (GTK_TYPE_STATUSBAR,
                                       "has-resize-grip", FALSE,
-                                      NULL);
+                                      (const char*) NULL);
     gtk_widget_set_name (GTK_WIDGET (window->statusbar), "no-shadow");
     gtk_box_pack_start (GTK_BOX (window->status_area),
                         GTK_WIDGET (window->statusbar),
@@ -506,7 +550,7 @@ moo_window_init (MooWindow *window)
 #else
                        "has-resize-grip", TRUE,
 #endif
-                       NULL);
+                       (const char*) NULL);
     gtk_widget_set_name (rg, "no-shadow");
     gtk_box_pack_end (GTK_BOX (window->status_area),
                       rg, FALSE, FALSE, 0);
@@ -547,6 +591,13 @@ moo_window_dispose (GObject *object)
             g_object_unref (window->priv->actions);
         }
 
+        g_slist_foreach (window->priv->global_accels,
+                         (GFunc) accel_entry_free, NULL);
+        g_slist_free (window->priv->global_accels);
+
+        if (window->priv->update_accels_idle)
+            g_source_remove (window->priv->update_accels_idle);
+
         g_free (window->priv->name);
         g_free (window->priv->id);
         g_free (window->priv->menubar_ui_name);
@@ -571,22 +622,130 @@ static gboolean
 moo_window_delete_event (GtkWidget      *widget,
                          G_GNUC_UNUSED GdkEventAny *event)
 {
-    gboolean result = FALSE;
+    MooCloseResponse result = FALSE;
     g_signal_emit_by_name (widget, "close", &result);
-    return result;
+    return TRUE;
 }
 
 
+static AccelEntry *
+accel_entry_new (guint            key,
+                 GdkModifierType  mods,
+                 GtkAction       *action)
+{
+    AccelEntry *entry = g_slice_new0 (AccelEntry);
+    entry->keyval = key;
+    entry->modifiers = mods;
+    entry->action = g_object_ref (action);
+    return entry;
+}
+
+static void
+accel_entry_free (AccelEntry *entry)
+{
+    if (entry)
+    {
+        g_object_unref (entry->action);
+        g_slice_free (AccelEntry, entry);
+    }
+}
+
+static gboolean
+update_accels (MooWindow *window)
+{
+    const GSList *l;
+
+    window->priv->update_accels_idle = 0;
+
+    g_slist_foreach (window->priv->global_accels, (GFunc) accel_entry_free, NULL);
+    g_slist_free (window->priv->global_accels);
+    window->priv->global_accels = NULL;
+
+    for (l = moo_action_collection_get_groups (window->priv->actions); l != NULL; l = l->next)
+    {
+        GtkActionGroup *group = l->data;
+        GList *actions;
+
+        actions = gtk_action_group_list_actions (group);
+
+        while (actions != NULL)
+        {
+            GtkAction *action = actions->data;
+            const char *accel_path = NULL;
+
+            if (MOO_IS_ACTION (action) &&
+                !_moo_action_get_no_accel (action))
+                    accel_path = gtk_action_get_accel_path (action);
+
+            if (accel_path && _moo_accel_prefs_get_global (accel_path))
+            {
+                const char *accel;
+                guint key;
+                GdkModifierType mods;
+
+                accel = _moo_get_accel (accel_path);
+
+                if (accel && accel[0] && _moo_accel_parse (accel, &key, &mods))
+                {
+                    AccelEntry *entry = accel_entry_new (key, mods, action);
+                    window->priv->global_accels =
+                        g_slist_prepend (window->priv->global_accels, entry);
+                }
+            }
+
+            actions = g_list_delete_link (actions, actions);
+        }
+    }
+
+    return FALSE;
+}
+
+static void
+accels_changed (MooWindow *window)
+{
+    if (!window->priv->update_accels_idle)
+        window->priv->update_accels_idle =
+            gdk_threads_add_idle ((GSourceFunc) update_accels, window);
+}
+
+static gboolean
+activate_global_accel (MooWindow   *window,
+                       GdkEventKey *event)
+{
+    guint keyval;
+    GdkModifierType mods;
+    GSList *l;
+
+    if (!window->priv->global_accels)
+        return FALSE;
+
+    moo_accel_translate_event (GTK_WIDGET (window), event, &keyval, &mods);
+
+    for (l = window->priv->global_accels; l != NULL; l = l->next)
+    {
+        AccelEntry *entry = l->data;
+
+        if (entry->keyval == keyval && entry->modifiers == mods)
+        {
+            gtk_action_activate (entry->action);
+            return TRUE;
+        }
+    }
+
+    return FALSE;
+}
+
 static gboolean
 moo_window_key_press_event (GtkWidget   *widget,
                             GdkEventKey *event)
 {
     MooWindow *window = MOO_WINDOW (widget);
 
-    if (window->priv->global_accels)
+    if (window->priv->global_accels_mode)
         return GTK_WIDGET_CLASS (moo_window_parent_class)->key_press_event (widget, event);
     else
-        return gtk_window_propagate_key_event (GTK_WINDOW (widget), event) ||
+        return activate_global_accel (window, event) ||
+               gtk_window_propagate_key_event (GTK_WINDOW (widget), event) ||
                gtk_window_activate_key (GTK_WINDOW (widget), event) ||
                /* GtkWindowClass would call two guys above again and then chain up
                 * to the parent, so it's a shortcut */
@@ -599,7 +758,7 @@ moo_window_set_global_accels (MooWindow *window,
                               gboolean   global)
 {
     g_return_if_fail (MOO_IS_WINDOW (window));
-    window->priv->global_accels = global != 0;
+    window->priv->global_accels_mode = global != 0;
 }
 
 
@@ -633,33 +792,33 @@ moo_window_save_size (MooWindow *window)
 {
     if (!window->priv->save_size_id)
         window->priv->save_size_id =
-                moo_idle_add ((GSourceFunc)save_size, window);
+                gdk_threads_add_idle ((GSourceFunc)save_size, window);
     return FALSE;
 }
 
 
+static MooCloseResponse
+moo_window_close_handler (G_GNUC_UNUSED MooWindow *window)
+{
+    return MOO_CLOSE_RESPONSE_CONTINUE;
+}
+
 gboolean
 moo_window_close (MooWindow *window)
 {
-    gboolean result = FALSE;
-    g_signal_emit_by_name (window, "close", &result);
-    if (!result)
+    MooCloseResponse response = MOO_CLOSE_RESPONSE_CONTINUE;
+
+    g_signal_emit_by_name (window, "close", &response);
+
+    if (response == MOO_CLOSE_RESPONSE_CANCEL)
+    {
+        return FALSE;
+    }
+    else
     {
         gtk_widget_destroy (GTK_WIDGET (window));
         return TRUE;
     }
-    else
-    {
-        return FALSE;
-    }
-}
-
-
-void
-moo_window_apply_prefs (MooWindow *window)
-{
-    g_return_if_fail (MOO_IS_WINDOW (window));
-    g_signal_emit (window, signals[APPLY_PREFS], 0);
 }
 
 
@@ -689,13 +848,13 @@ moo_window_set_property (GObject      *object,
     {
         case PROP_TOOLBAR_UI_NAME:
             name = g_value_get_string (value);
-            MOO_ASSIGN_STRING (window->priv->toolbar_ui_name, name ? name : "");
+            MOO_ASSIGN_STRING (window->priv->toolbar_ui_name, MOO_NZS (name));
             g_object_notify (object, "toolbar-ui-name");
             break;
 
         case PROP_MENUBAR_UI_NAME:
             name = g_value_get_string (value);
-            MOO_ASSIGN_STRING (window->priv->menubar_ui_name, name ? name : "");
+            MOO_ASSIGN_STRING (window->priv->menubar_ui_name, MOO_NZS (name));
             g_object_notify (object, "menubar-ui-name");
             break;
 
@@ -755,6 +914,14 @@ moo_window_get_property (GObject      *object,
             g_value_set_object (value, window->priv->ui_xml);
             break;
 
+        case PROP_MENUBAR:
+            g_value_set_object (value, window->menubar);
+            break;
+
+        case PROP_TOOLBAR:
+            g_value_set_object (value, window->toolbar);
+            break;
+
         case PROP_ACTIONS:
             g_value_set_object (value, moo_window_get_actions (window));
             break;
@@ -808,10 +975,25 @@ moo_window_get_property (GObject      *object,
 }
 
 
+void
+moo_window_set_title (MooWindow  *window,
+                      const char *title)
+{
+    const char *old_title;
+
+    g_return_if_fail (window != NULL);
+
+    old_title = gtk_window_get_title (GTK_WINDOW (window));
+
+    if (!moo_str_equal (title, old_title))
+        gtk_window_set_title (GTK_WINDOW (window), MOO_NZS (title));
+}
+
+
 static void
 moo_window_update_toolbar (MooWindow *window)
 {
-    MooUIXML *xml;
+    MooUiXml *xml;
     GtkToolbarStyle style;
     char *ui_name;
     MooActionCollection *actions;
@@ -825,7 +1007,7 @@ moo_window_update_toolbar (MooWindow *window)
 
     if (window->toolbar)
     {
-        MooUIXML *old_xml;
+        MooUiXml *old_xml;
         char *old_name;
 
         old_xml = g_object_get_data (G_OBJECT (window->toolbar), "moo-window-ui-xml");
@@ -856,13 +1038,15 @@ moo_window_update_toolbar (MooWindow *window)
 
     style = get_toolbar_style (window);
     gtk_toolbar_set_style (GTK_TOOLBAR (MOO_WINDOW(window)->toolbar), style);
+
+    g_object_notify (G_OBJECT (window), "toolbar");
 }
 
 
 static void
 moo_window_update_menubar (MooWindow *window)
 {
-    MooUIXML *xml;
+    MooUiXml *xml;
     char *ui_name;
     MooActionCollection *actions;
 
@@ -875,7 +1059,7 @@ moo_window_update_menubar (MooWindow *window)
 
     if (window->menubar)
     {
-        MooUIXML *old_xml;
+        MooUiXml *old_xml;
         char *old_name;
 
         old_xml = g_object_get_data (G_OBJECT (window->menubar), "moo-window-ui-xml");
@@ -903,6 +1087,8 @@ moo_window_update_menubar (MooWindow *window)
 
     gtk_box_pack_start (GTK_BOX (window->priv->menubar_holder),
                         window->menubar, FALSE, FALSE, 0);
+
+    g_object_notify (G_OBJECT (window), "menubar");
 }
 
 
@@ -917,8 +1103,9 @@ moo_window_update_ui (MooWindow *window)
 static void
 moo_window_shortcuts_prefs_dialog (MooWindow *window)
 {
-    _moo_accel_prefs_dialog_run (moo_window_get_actions (window),
-                                 GTK_WIDGET (window));
+    if (_moo_accel_prefs_dialog_run (moo_window_get_actions (window),
+                                     GTK_WIDGET (window)))
+        accels_changed (window);
 }
 
 
@@ -977,7 +1164,7 @@ toolbar_style_toggled (MooWindow            *window,
     GtkToolbarStyle style = GPOINTER_TO_INT (data);
     if (window->toolbar)
         gtk_toolbar_set_style (GTK_TOOLBAR (window->toolbar), style);
-    moo_prefs_set_enum (setting (window, PREFS_TOOLBAR_STYLE), style);
+    moo_prefs_set_int (setting (window, PREFS_TOOLBAR_STYLE), style);
 }
 
 
@@ -1054,9 +1241,8 @@ init_prefs (MooWindow *window)
     moo_prefs_new_key_bool (setting (window, PREFS_SHOW_TOOLBAR), TRUE);
     moo_prefs_new_key_bool (setting (window, PREFS_SHOW_MENUBAR), TRUE);
     moo_prefs_new_key_bool (setting (window, PREFS_SHOW_STATUSBAR), TRUE);
-    moo_prefs_new_key_enum (setting (window, PREFS_TOOLBAR_STYLE),
-                            GTK_TYPE_TOOLBAR_STYLE,
-                            get_toolbar_style_gtk (window));
+    moo_prefs_new_key_int (setting (window, PREFS_TOOLBAR_STYLE),
+                           get_toolbar_style_gtk (window));
 
     moo_prefs_create_key (setting (window, PREFS_MAXIMIZED), MOO_PREFS_STATE, G_TYPE_BOOLEAN, FALSE);
     moo_prefs_create_key (setting (window, PREFS_WIDTH), MOO_PREFS_STATE, G_TYPE_INT, 800);
@@ -1066,7 +1252,7 @@ init_prefs (MooWindow *window)
 static GtkToolbarStyle
 get_toolbar_style (MooWindow *window)
 {
-    GtkToolbarStyle s = moo_prefs_get_enum (setting (window, PREFS_TOOLBAR_STYLE));
+    GtkToolbarStyle s = moo_prefs_get_int (setting (window, PREFS_TOOLBAR_STYLE));
     g_return_val_if_fail (s < N_STYLES, GTK_TOOLBAR_ICONS);
     return s;
 }
@@ -1078,9 +1264,12 @@ get_toolbar_style (MooWindow *window)
 /* Actions
  */
 
-#define MOO_WINDOW_NAME_QUARK        (get_quark__(0))
-#define MOO_WINDOW_ID_QUARK          (get_quark__(1))
-#define MOO_WINDOW_ACTIONS_QUARK_    (get_quark__(2))
+#define MOO_WINDOW_NAME_QUARK (moo_window_name_quark ())
+MOO_DEFINE_QUARK_STATIC (moo-window-name, moo_window_name_quark)
+#define MOO_WINDOW_ID_QUARK (moo_window_id_quark ())
+MOO_DEFINE_QUARK_STATIC (moo-window-id, moo_window_id_quark)
+#define MOO_WINDOW_ACTIONS_QUARK (moo_window_actions_quark ())
+MOO_DEFINE_QUARK_STATIC (moo-window-actions, moo_window_actions_quark)
 
 typedef struct {
     GHashTable *groups; /* name -> display_name */
@@ -1103,24 +1292,6 @@ typedef struct {
 } ActionInfo;
 
 
-static GQuark
-get_quark__ (guint n)
-{
-    static GQuark q[3];
-
-    g_assert (n < 3);
-
-    if (!q[0])
-    {
-        q[0] = g_quark_from_static_string ("moo-window-name");
-        q[1] = g_quark_from_static_string ("moo-window-id");
-        q[2] = g_quark_from_static_string ("moo-window-actions");
-    }
-
-    return q[n];
-}
-
-
 static ActionInfo*
 action_info_new (MooActionFactory *action,
                  const char       *group,
@@ -1176,19 +1347,19 @@ action_store_new (void)
 static ActionStore *
 type_get_store (GType type)
 {
-    return g_type_get_qdata (type, MOO_WINDOW_ACTIONS_QUARK_);
+    return g_type_get_qdata (type, MOO_WINDOW_ACTIONS_QUARK);
 }
 
 
 static ActionStore *
 type_ensure_store (GType type)
 {
-    ActionStore *store = g_type_get_qdata (type, MOO_WINDOW_ACTIONS_QUARK_);
+    ActionStore *store = g_type_get_qdata (type, MOO_WINDOW_ACTIONS_QUARK);
 
     if (!store)
     {
         store = action_store_new ();
-        g_type_set_qdata (type, MOO_WINDOW_ACTIONS_QUARK_, store);
+        g_type_set_qdata (type, MOO_WINDOW_ACTIONS_QUARK, store);
     }
 
     return store;
@@ -1537,7 +1708,7 @@ moo_window_class_remove_group (MooWindowClass *klass,
 }
 
 
-MooUIXML*
+MooUiXml*
 moo_window_get_ui_xml (MooWindow          *window)
 {
     g_return_val_if_fail (MOO_IS_WINDOW (window), NULL);
@@ -1547,7 +1718,7 @@ moo_window_get_ui_xml (MooWindow          *window)
 
 void
 moo_window_set_ui_xml (MooWindow          *window,
-                       MooUIXML           *xml)
+                       MooUiXml           *xml)
 {
     g_return_if_fail (MOO_IS_WINDOW (window));
     g_return_if_fail (!xml || MOO_IS_UI_XML (xml));
@@ -1696,6 +1867,8 @@ moo_window_add_action (MooWindow  *window,
         if (_moo_action_get_connect_accel (action))
             gtk_action_connect_accelerator (action);
 
+        accels_changed (window);
+
         g_free (accel_path);
     }
 }
@@ -1897,7 +2070,7 @@ collect_params_and_props (guint              n_callback_args,
 
         if (!action_factory)
         {
-            g_warning ("%s: error in moo_action_factory_new_a()", G_STRLOC);
+            g_warning ("oops");
             goto error;
         }
 
@@ -1979,7 +2152,7 @@ moo_window_class_new_action (MooWindowClass     *klass,
     if (!success)
     {
         if (error)
-            g_critical ("%s: %s", G_STRLOC, error);
+            g_critical ("%s", error);
         g_free (error);
         return;
     }
@@ -2017,7 +2190,7 @@ _moo_window_class_new_action_callback (MooWindowClass *klass,
     if (!success)
     {
         if (error)
-            g_critical ("%s: %s", G_STRLOC, error);
+            g_critical ("%s", error);
         g_free (error);
         return;
     }
